import { DatePipe } from '@angular/common';
import { Component, OnInit } from '@angular/core';
import { ColDef, ICellRendererParams } from 'ag-grid-community';
import { finalize } from 'rxjs/operators';
import { NotificationService } from '../../../services/notification.service';
import { AdminStructure } from '../../models/adminStructureList.interface';
import { AdminService } from '../../services/admin.service';
import { ButtonCellRendererComponent } from '../button-cell-renderer/button-cell-renderer.component';

@Component({
  selector: 'app-admin-structures-list',
  templateUrl: './admin-structures-list.component.html',
  styleUrls: ['../../admin.scss', 'admin-structures-list.component.scss'],
})
export class AdminStructuresListComponent implements OnInit {
  public structuresClaimed: AdminStructure[];
  public structuresInClaim: AdminStructure[];
  public structuresToClaim: AdminStructure[];
  public structuresIncomplete: AdminStructure[];
  public isLoading = true;
  public rowHeight = 48;
  public filter = 'all';

  constructor(
    private adminService: AdminService,
    private datePipe: DatePipe,
    private notificationService: NotificationService,
  ) {}

  public filterOptions = [
    { label: 'Toutes les structures', value: 'all' },
    { label: 'Données manquantes', value: 'incomplete' },
    { label: 'En cours de revendication', value: 'inClaim' },
    { label: 'À revendiquer', value: 'toClaim' },
  ];

  public columnDefs: ColDef<AdminStructure>[] = [
    {
      headerName: 'Structure',
      cellRenderer: (params): string => this.renderLink(params.data),
      comparator(_, __, nodeA, nodeB): number {
        return nodeA.data.structureName.toLowerCase() > nodeB.data.structureName.toLowerCase() ? -1 : 1;
      },
      cellClass: 'cell-link',
      sortable: true,
      flex: 3,
    },
    {
      headerName: 'Date de mise à jour',
      field: 'updatedAt',
      valueFormatter: ({ value }): string => this.datePipe.transform(value, 'mediumDate'),
      cellClass: (params): string => (params.data.isOutdated ? 'red' : ''),
      flex: 1,
      sortable: true,
    },
    {
      headerName: 'Dernière relance par mail',
      field: 'lastUpdateMail',
      valueFormatter: ({ value }): string => this.datePipe.transform(value, 'mediumDate'),
      flex: 1,
      sortable: true,
    },
    {
      headerName: 'Relancer',
      cellRenderer: ButtonCellRendererComponent,
      cellRendererParams: { label: 'Relancer' },
      flex: 1,
      sortable: false,
      onCellClicked: this.handleRelance.bind(this),
    },
  ];

  public gridSections = [];
  public filteredSections = [];

  ngOnInit(): void {
    this.adminService
      .getAllStructureAdmin()
      .pipe(finalize(() => (this.isLoading = false)))
      .subscribe((response) => {
        this.structuresClaimed = response.claimed;
        this.structuresInClaim = response.inClaim;
        this.structuresToClaim = response.toClaim;
        this.structuresIncomplete = response.incomplete;

        this.gridSections = [
          {
            filter: 'incomplete',
            title: 'Structures avec des données manquantes',
            data: this.structuresIncomplete,
            cssClass: 'red',
          },
          {
            filter: 'inClaim',
            title: 'Structures en cours de revendication',
            data: this.structuresInClaim,
            cssClass: '',
          },
          {
            filter: 'toClaim',
            title: 'Structures à revendiquer',
            data: this.structuresToClaim,
            cssClass: '',
          },
          {
            filter: 'claimed',
            title: 'Structures revendiquées',
            data: this.structuresClaimed,
            cssClass: '',
          },
        ];

        this.updateFilteredSections();
      });
  }

  setFilter(filter: string | boolean): void {
    this.filter = filter as string;
    this.updateFilteredSections();
  }

  updateFilteredSections(): void {
    if (this.filter === 'all') {
      this.filteredSections = this.gridSections;
    } else {
      this.filteredSections = this.gridSections.filter((section) => section.filter === this.filter);
    }
  }

  public getGridContext(filterName: string): { context: { gridId: string } } {
    return { context: { gridId: filterName } };
  }

  private renderLink(structure: AdminStructure): string {
    return `<a href="/acteurs?structure=${structure.permalink}" target="_blank">${structure.structureName}</a>`;
  }

  private handleRelance(params: ICellRendererParams): void {
    this.adminService.updateLastUpdateMail(params.data.structureId).subscribe({
      next: () => {
        params.api.getRowNode(String(params.rowIndex)).setDataValue('lastUpdateMail', new Date());
        this.notificationService.showSuccess('Relance effectuée');
      },
      error: (error) => {
        console.error('Error updating last update mail:', error);
        this.notificationService.showError('Erreur: ' + (error.message || 'unknown error'));
      },
    });
  }
}
