import { Component, ElementRef, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { forkJoin, lastValueFrom } from 'rxjs';
import { SearchService } from '../../structure-list/services/search.service';
import { Utils } from '../../utils/utils';
import { TypeModal } from '../enums/TypeModal.enum';
import { SearchQuery } from '../models/searchQuery.model';

@Component({
  selector: 'app-annuaire-header',
  templateUrl: './annuaire-header.component.html',
  styleUrls: ['./annuaire-header.component.scss'],
})
export class AnnuaireHeaderComponent implements OnInit, OnChanges {
  @Input() shouldResetFilters = 0;
  @Output() searchEvent = new EventEmitter<SearchQuery>();

  public utils = new Utils();
  public modalTypeOpened: TypeModal;
  public employersFiltersActive = false;
  public jobsFiltersActive = false;
  public jobTypes: string[] = [];
  public employerTypes: string[] = [];
  public TypeModal = TypeModal;

  public searchInput = '';
  public jobFilterChecked: string[] = [];
  public employerFilterChecked: string[] = [];
  public keyboardEvent = false;

  constructor(
    private activatedRoute: ActivatedRoute,
    private router: Router,
    public searchService: SearchService,
    private elementRef: ElementRef,
  ) {}

  async ngOnInit(): Promise<void> {
    this.loadQueryParams();
    // Will store the different categories
    await this.loadFilters();

    if (this.searchService.previousResult$.getValue().docs.length === 0) {
      this.applyFilter();
    }
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.shouldResetFilters?.currentValue !== 0) {
      this.resetFilters();
    }
  }

  private loadQueryParams(): void {
    const queryParams = this.activatedRoute.snapshot.queryParams;
    this.searchInput = queryParams['search'] ?? '';
    this.jobFilterChecked = queryParams['jobs']?.split('|') ?? [];
    this.employerFilterChecked = queryParams['employers']?.split('|') ?? [];
    this.searchService.checkedFilterList = [...this.jobFilterChecked, ...this.employerFilterChecked];
    this.searchService.annuaireSearchQuery = {
      search: this.searchInput,
      page: Number(queryParams['page']) || 1,
      jobFilters: this.jobFilterChecked,
      employerFilters: this.employerFilterChecked,
    };
  }

  public onSearchSubmitted(value: string): void {
    this.searchInput = value;
    this.applyFilter();
  }

  private updateQueryParams(): void {
    const queryParams: Record<string, string> = {};

    if (this.searchInput.trim()) {
      queryParams.search = this.searchInput.trim();
    }

    if (this.jobFilterChecked.length) {
      queryParams.jobs = this.jobFilterChecked.join('|');
    }

    if (this.employerFilterChecked.length) {
      queryParams.employers = this.employerFilterChecked.join('|');
    }

    // Delay to avoid error in console: "NG0100: ExpressionChangedAfterItHasBeenCheckedError"
    setTimeout(
      () =>
        this.router.navigate([], {
          relativeTo: this.activatedRoute,
          queryParams: queryParams,
        }),
      1,
    );
  }

  public applyFilter(page = 1): void {
    this.searchEvent.emit({
      search: this.searchInput.trim(),
      page: page,
      jobFilters: this.jobFilterChecked,
      employerFilters: this.employerFilterChecked,
    });

    this.updateQueryParams();
  }

  /** Get the categories for each modal type */
  private async loadFilters(): Promise<void> {
    const { job, employer } = await lastValueFrom(
      forkJoin({
        job: this.searchService.getJobsGroups(),
        employer: this.searchService.getEmployers(),
      }),
    );
    this.jobTypes = job;
    this.employerTypes = employer;
    this.countCheckedFilters();
  }

  private splitFilters(checkedFilterList: string[]): void {
    this.employerFilterChecked = checkedFilterList.filter((filter) => this.employerTypes.includes(filter));
    this.jobFilterChecked = checkedFilterList.filter((filter) => this.jobTypes.includes(filter));
  }

  public fetchResults(checkedFilters: string[]): void {
    this.searchService.checkedFilterList = checkedFilters;
    this.splitFilters(checkedFilters);
    this.closeModal();
    this.applyFilter();
  }

  public getModalCategory(): string[] {
    switch (this.modalTypeOpened) {
      case TypeModal.jobs:
        return this.jobTypes;
      case TypeModal.employers:
        return this.employerTypes;
      default:
        return null;
    }
  }

  /** Open the modal and display the list according to the right filter button */
  public openModal(modalType: TypeModal): void {
    this.modalTypeOpened = this.modalTypeOpened === modalType ? undefined : modalType;

    // Accessibility: when navigating with keyboard and opening a filter modal, send focus to the first focusable element of the opened modal
    if (this.keyboardEvent) {
      setTimeout(() => {
        this.utils.setFocusOnElement(this.elementRef, `.modalContent .collapse-header, .modalContent input`);
      }, 0);
    }
  }

  // When filters and their modal are in the same component, we can remove onKeyboardNavOnFilters, setFocusOnOpenedModal, and setFocusOnFilters.
  // because the focus will then flow normally between the filter and the modal
  public onKeyboardNavOnFilters(event: KeyboardEvent): void {
    switch (event.key) {
      case 'ArrowUp':
      case 'ArrowDown':
      case 'Tab':
        this.keyboardEvent = true;
        break;
    }
  }

  public closeModal(): void {
    // Accessibility: when navigating with keyboard and closing a filter modal, send focus back to filters
    if (this.keyboardEvent) {
      this.utils.setFocusOnElement(this.elementRef, '#modal' + this.modalTypeOpened + ' button:first-of-type');
    }
    this.modalTypeOpened = undefined;
    this.countCheckedFilters();
  }

  /** Check if some modules is checked on filter and store number of modules checked */
  private countCheckedFilters(): void {
    this.jobsFiltersActive = this.searchService.checkedFilterList.some((filter) => this.jobTypes.includes(filter));
    this.employersFiltersActive = this.searchService.checkedFilterList.some((filter) =>
      this.employerTypes.includes(filter),
    );
  }

  public resetFilters(): void {
    this.searchInput = '';
    this.searchService.checkedFilterList = [];
    this.employersFiltersActive = false;
    this.jobsFiltersActive = false;
    this.employerFilterChecked = [];
    this.jobFilterChecked = [];

    // Delay to avoid error in console: "NG0100: ExpressionChangedAfterItHasBeenCheckedError"
    setTimeout(
      () =>
        this.router.navigate([], {
          relativeTo: this.activatedRoute,
          queryParams: {},
        }),
      1,
    );
  }

  public removeFilter(filter: string): void {
    const index = this.searchService.checkedFilterList.findIndex((checkedFilter: string) => checkedFilter === filter);
    if (index !== -1) {
      this.searchService.checkedFilterList.splice(index, 1);
      this.splitFilters(this.searchService.checkedFilterList);
      this.countCheckedFilters();
      this.updateQueryParams();
      this.applyFilter();
    }
  }
}
