import { Component, HostListener, OnInit, ViewChild } from '@angular/core';
import {
  GuardsCheckStart,
  NavigationCancel,
  NavigationEnd,
  NavigationError,
  NavigationStart,
  Router,
} from '@angular/router';
import { MatomoInitializerService } from 'ngx-matomo-client';
import { HeaderComponent } from './header/header.component';
import { ProfileService } from './profile/services/profile.service';
import { AuthService } from './services/auth.service';
import { ConfigService } from './services/config.service';
import { RouterListenerService } from './services/routerListener.service';
import { UpdateService } from './services/update.service';
import { PrintService } from './shared/service/print.service';
import { WindowScrollService } from './shared/service/windowScroll.service';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss'],
})
export class AppComponent implements OnInit {
  public loading = true;

  @ViewChild(HeaderComponent) headerComponent!: HeaderComponent;

  // Listener, keyboard shortcuts
  @HostListener('window:keydown', ['$event'])
  handleKeyboardEvent(event: KeyboardEvent): void {
    // 'esc' to go to the header and escape any focus trap
    if (event.key === 'Escape') {
      this.headerComponent.focusLogo();
      event.preventDefault();
    }
  }

  constructor(
    public printService: PrintService,
    private authService: AuthService,
    private profilService: ProfileService,
    private routerListener: RouterListenerService,
    private updateService: UpdateService,
    private router: Router,
    private configService: ConfigService,
    private matomoInitializer: MatomoInitializerService,
    private windowScrollService: WindowScrollService,
  ) {
    if (this.authService.isLoggedIn()) {
      this.profilService.getProfile();
    }
    this.routerListener.loadRouting();
    // handle pwa update
    this.updateService.subscribeUpdate();

    this.router.events.subscribe((event) => {
      switch (true) {
        case event instanceof NavigationStart: {
          this.loading = true;
          break;
        }

        case event instanceof GuardsCheckStart:
        case event instanceof NavigationEnd:
        case event instanceof NavigationCancel: {
          this.loading = false;
          break;
        }
        case event instanceof NavigationError: {
          setTimeout(() => {
            this.loading = false;
          }, 100);
          break;
        }
        default: {
          break;
        }
      }
    });
  }

  ngOnInit(): void {
    /**
     * Reset scroll to top for article reading
     */
    this.router.events.subscribe((evt) => {
      if (!(evt instanceof NavigationEnd)) {
        return;
      }
      document.getElementsByClassName('app-body')[0].scrollTo(0, 0);
    });

    this.configService.getConfig().then((config) => {
      if (config.matomoTrackerUrl && config.matomoSiteId) {
        this.matomoInitializer.initializeTracker({
          trackerUrl: config.matomoTrackerUrl,
          siteId: config.matomoSiteId,
        });
      }
    });
  }

  public onScroll(event): void {
    this.windowScrollService.scrollY.next(event.target.scrollTop);

    // Check if scrolled to bottom: visible height + pixel scrolled >= total height
    if (event.target.offsetHeight + event.target.scrollTop >= event.target.scrollHeight) {
      this.windowScrollService.scrolledToBottom = true;
    }
  }
}
