import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { Meta } from '@angular/platform-browser';
import { ActivatedRoute } from '@angular/router';
import * as _ from 'lodash';
import { GeoJson } from '../map/models/geojson.model';
import { Structure } from '../models/structure.model';
import { ProfileService } from '../profile/services/profile.service';
import { GeojsonService } from '../services/geojson.service';
import { StructureService } from '../services/structure.service';
import { Filter } from '../structure-list/models/filter.model';
import { CustomRegExp } from '../utils/CustomRegExp';

@Component({
  selector: 'app-carto',
  templateUrl: './carto.component.html',
  styleUrls: ['./carto.component.scss'],
})
export class CartoComponent implements OnInit {
  @Input() public isOrientationForm = false;
  @Input() public isOrientationRdv = false;
  @Input() public filters: Filter[] = [];
  @Input() public userLatitude: number = null;
  @Input() public userLongitude: number = null;
  @Input() public structuresSelected: Structure[] = [];
  @Output() structureSelection = new EventEmitter<any>();
  @Output() structureSelectionRDV = new EventEmitter<any>();

  public structures: Structure[] = [];
  public isLoadingStructures = true;
  public currentLocation: GeoJson;
  public displayMarkerId: string;
  public selectedMarkerId: string;
  public geolocation = false;
  public currentStructure: Structure;
  public isMapPhone = false;
  public searchedValue = null;
  public userLocate = null;

  constructor(
    private structureService: StructureService,
    private geoJsonService: GeojsonService,
    private activatedRoute: ActivatedRoute,
    private profileService: ProfileService,
    private meta: Meta,
  ) {}

  ngOnInit(): void {
    if (!this.activatedRoute.snapshot.queryParamMap.get('query')) {
      // Set user location if provided
      if (this.userLatitude && this.userLongitude) {
        this.userLocate = [this.userLongitude, this.userLatitude];
        this.getAddress(this.userLongitude, this.userLatitude);
      }
      if (navigator.geolocation) {
        this.getLocation();
      }
    }

    this.meta.updateTag({
      name: 'description',
      content: 'Recense tous les lieux, accompagnements et ateliers de médiation numérique de la Métropole de Lyon.',
    });
  }

  public getStructures(filters: Filter[]): void {
    this.isLoadingStructures = true;
    let mustLoadStructures = false;
    let sortByDistance = false;

    const queryString = _.find(filters, { name: 'query' });
    if (queryString) {
      this.searchedValue = queryString.value;
      if (this.isLocationRequest(queryString.value)) {
        this.getCoordByAddress(queryString.value).then((res) => {
          this.currentLocation = res;
          this.updateStructuresDistance(
            this.structures,
            this.currentLocation.geometry.getLon(),
            this.currentLocation.geometry.getLat(),
          );
          this.isLoadingStructures = false;
        });
      } else {
        mustLoadStructures = true;
        sortByDistance = false;
      }
    } else {
      this.searchedValue = null;
      mustLoadStructures = true;
      sortByDistance = true;
    }

    if (mustLoadStructures) {
      // For orientation rdv, structure offers will be ignored: we filter only on personalOffers of the structure with appointment
      this.structureService.getStructures(filters, 'search', this.isOrientationRdv).subscribe((structures) => {
        if (structures) {
          if (this.isOrientationRdv) structures = structures.filter((structure) => structure.hasUserWithAppointmentDN);
          this.updateStructuresDistance(structures, this.userLongitude, this.userLatitude, sortByDistance);
        } else {
          this.structures = [];
        }
        this.isLoadingStructures = false;
      });
    }
  }

  /**
   * Update structure distance according to user actual position.
   * @param structures structures data to update
   * @param lon user longitude
   * @param lat user latitude
   * @param sortByDistance if set to `true`, structures data is sort by distance. Default value is `true`
   */
  private updateStructuresDistance(structures: Structure[], lon: number, lat: number, sortByDistance = true): void {
    Promise.all(
      structures.map(async (structure) => {
        if (lon && lat) {
          structure = this.getStructurePosition(structure, lon, lat);
        }
        return structure;
      }),
    ).then((structureList) => {
      if (sortByDistance) {
        structureList = _.sortBy(structureList, ['distance']);
      }
      this.structures = structureList;
    });
  }

  /**
   * Retrieve GeoJson for a given address
   * @param address string
   */
  private getCoordByAddress(address: string): Promise<GeoJson> {
    return new Promise((resolve) => {
      this.geoJsonService.getCoord(address, '', '69000').subscribe((res) => {
        resolve(res);
      });
    });
  }

  /**
   * Check with a regex that an address is request
   * @param value string
   */
  private isLocationRequest(value: string): boolean {
    if (value.match(CustomRegExp.LOCATION)) {
      return true;
    }
    return false;
  }

  /**
   * Get structures positions and add marker corresponding to those positons on the map
   * @param structure Structure
   * @param lon number
   * @param lat number
   */
  private getStructurePosition(structure: Structure, lon: number, lat: number): Structure {
    structure.distance = parseInt(
      this.geoJsonService.getDistance(structure.getLat(), structure.getLon(), lat, lon, 'M'),
      10,
    );
    return structure;
  }

  public getLocation(): void {
    navigator.geolocation.getCurrentPosition((position) => {
      this.geolocation = true;
      if (!this.userLongitude && !this.userLatitude) {
        this.userLongitude = position.coords.longitude;
        this.userLatitude = position.coords.latitude;
      }
      this.getAddress(position.coords.longitude, position.coords.latitude);
      this.updateStructuresDistance(this.structures, this.userLongitude, this.userLatitude);
    });
  }

  /**
   * Get an address by coord
   * @param longitude number
   * @param latitude number
   */
  private getAddress(longitude: number, latitude: number): void {
    this.geoJsonService.getAddressByCoord(longitude, latitude).subscribe(
      (location) => {
        this.currentLocation = location;
      },
      (err) => {
        throw new Error(err);
      },
    );
  }

  public setMapMarkerId(event: string): void {
    this.displayMarkerId = event;
  }

  public setSelectedMarkerId(id: string): void {
    this.selectedMarkerId = id;
  }

  public showDetailStructure(structure: Structure): void {
    this.currentStructure = new Structure(structure);
  }

  public switchMapList(): void {
    this.isMapPhone = !this.isMapPhone;
  }

  public get isAdmin(): boolean {
    return this.profileService.isAdmin();
  }

  public selectStructure(structures: Structure[]): void {
    this.structuresSelected = structures;
    this.structureSelection.emit(this.structuresSelected);
  }

  public selectStructureRDV(structure: Structure): void {
    this.structureSelectionRDV.emit(structure);
  }
}
