import { AfterViewInit, ChangeDetectorRef, Component, OnInit } from '@angular/core';
import { UntypedFormControl, UntypedFormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, NavigationEnd, Router } from '@angular/router';
import { forkJoin, of } from 'rxjs';
import { catchError, map } from 'rxjs/operators';
import { PersonalOffer } from '../../models/personalOffer.model';
import { Structure } from '../../models/structure.model';
import { StructureWithOwners } from '../../models/structureWithOwners.model';
import { User } from '../../models/user.model';
import { ProfileService } from '../../profile/services/profile.service';
import { NotificationService } from '../../services/notification.service';
import { PersonalOfferService } from '../../services/personal-offer.service';
import { StructureService } from '../../services/structure.service';
import { UserService } from '../../services/user.service';
import { MustMatch } from '../../shared/validator/form';
import { CustomRegExp } from '../../utils/CustomRegExp';
import { FormUtils } from '../../utils/formUtils';
import { CanExitResolver, Utils } from '../../utils/utils';
import { stepType } from '../step.type';
import { accountFormStep } from './account-form/accountFormStep.enum';
import { formType, formTypeParam } from './formType.enum';
import { personalOfferFormStep } from './personal-offer-form/personalOfferFormStep.enum';
import { profileFormStep } from './profile-form/profileFormStep.enum';
import { structureFormStep } from './structure-form/structureFormStep.enum';

@Component({
  selector: 'app-form-view',
  templateUrl: './form-view.component.html',
  styleUrls: ['./form-view.component.scss'],
})
export class FormViewComponent implements OnInit, AfterViewInit {
  public routeParam: string;
  public formType = formType;
  public formTypeParam = formTypeParam;
  public currentPage: stepType;
  public currentFormType: formType;
  public currentForm: UntypedFormGroup;
  public formUtils = new FormUtils();
  // Account Form
  public accountForm: UntypedFormGroup;
  // Profile Form
  public profileForm: UntypedFormGroup;
  public isPersonalOfferProfile = false;

  // Structure from
  public structureForm: UntypedFormGroup;
  public editForm: UntypedFormGroup;
  public structure: Structure;
  public isEditMode = false;
  public structureFormStep = structureFormStep;

  // Personal Offers Form
  public personalOfferForm: UntypedFormGroup;
  public hasOtherPersonalOffer = false;

  // Page and progress var
  public previousSteps = 0;
  public nbSteps: number;

  // Collapse var
  public showWebsite: boolean;
  public showSocialNetwork: boolean;

  // Modal canExit var
  public showConfirmationModal = false;
  private resolve: CanExitResolver;

  public profile: User;
  public isAccountMode = false;
  public isJoinMode = false;
  public claimStructure = false;
  public linkedStructureId;
  public structureWithOwners: StructureWithOwners;
  public isPageValid = false;
  public hoursForm: UntypedFormGroup;
  public isRegisterNewMember = false;

  constructor(
    private cdRef: ChangeDetectorRef,
    private notificationService: NotificationService,
    private personalOfferService: PersonalOfferService,
    private profileService: ProfileService,
    private route: ActivatedRoute,
    private router: Router,
    private structureService: StructureService,
    private utils: Utils,
    private usersService: UserService,
  ) {}

  ngAfterViewInit(): void {
    this.cdRef.detectChanges();
  }

  async ngOnInit(): Promise<void> {
    // Get structure from history (this is used to create personal offer from profile)
    if (history.state.structure) {
      this.structure = history.state.structure;
      this.isPersonalOfferProfile = true;
    }
    this.routeParam = this.router.routerState.snapshot.url.split('/')[2];

    this.initPage();
    this.profileService.getProfile().then((user: User) => {
      this.profile = user;

      if (!this.isEditMode && this.profile) {
        if (this.profile.structuresLink.length) {
          // if register a new user as a new member structure, no structure to choose
          this.isRegisterNewMember = true;
        }
      }
    });
    // Check if it's a new structure or edit structure
    this.route.data.subscribe((data) => {
      if (data.structure) {
        this.isEditMode = true;
        this.structure = data.structure;
        this.editForm = this.formUtils.createStructureForm(data.structure, this.isEditMode);
        this.structureForm = this.editForm;
        this.hoursForm = this.formUtils.createHoursForm(data.structure);
      }
    });

    this.router.events.subscribe((evt) => {
      if (!(evt instanceof NavigationEnd)) {
        return;
      }
      this.routeParam = this.router.routerState.snapshot.url.split('/')[2];

      this.initPage();
    });
  }

  private initPage(): void {
    const profileFormSteps = Object.keys(profileFormStep).length / 2 - 1;
    const structureFormSteps = Object.keys(structureFormStep).length / 2 - 1;
    const personalOfferFormSteps = Object.keys(personalOfferFormStep).length / 2 - 1;
    const totalFormSteps = profileFormSteps + structureFormSteps + personalOfferFormSteps;
    if (this.routeParam === formTypeParam.account) {
      this.nbSteps = 3;
      this.currentPage = accountFormStep.accountInfo;
      this.currentFormType = formType.account;
      this.createAccountForm();
      this.currentForm = this.accountForm;
    }
    if (this.routeParam === formTypeParam.register) {
      this.nbSteps = 3;
      this.currentPage = accountFormStep.accountInfo;
      this.currentFormType = formType.account;
      this.route.data.subscribe((data) => {
        if (data.user) {
          this.createAccountForm(data.user.email);
          this.linkedStructureId = data.user.structuresLink;
          this.currentForm = this.accountForm;
          this.isAccountMode = true;
        }
      });
    }
    if (this.routeParam === formTypeParam.profile) {
      this.nbSteps = totalFormSteps;
      this.currentPage = profileFormStep.profileBeginningInfo;
      this.currentFormType = formType.profile;
      this.createProfileForm();
      this.currentForm = this.profileForm;
    }
    if (this.routeParam === formTypeParam.structure) {
      if (!this.isEditMode) {
        this.nbSteps = structureFormSteps;
        this.currentPage = structureFormStep.structureChoice;
        this.currentFormType = formType.structure;
        this.structure = new Structure();
        this.structureForm = this.formUtils.createStructureForm(this.structure);
        this.currentForm = this.structureForm;
        // Init hours form
        this.hoursForm = this.formUtils.createHoursForm(this.structure);
        // Two pages don't increment the counter when coming from the carto
        if (this.previousSteps === 0) {
          this.nbSteps = structureFormSteps - 2;
        }
      }
    }
    if (this.routeParam === formTypeParam.personaloffer) {
      this.nbSteps = personalOfferFormSteps;
      this.currentPage = personalOfferFormStep.personalOfferAccompaniment;
      this.currentFormType = formType.personaloffer;
      this.createPersonalOfferForm(new PersonalOffer());
      this.currentForm = this.personalOfferForm;
    }
  }

  public displayedCurrentPage(): number {
    return this.previousSteps + this.currentPage - 1;
  }

  public updateHours(form: UntypedFormGroup): void {
    this.hoursForm = form;
  }

  private createAccountForm(email?: string): void {
    this.accountForm = new UntypedFormGroup(
      {
        email: new UntypedFormControl(email ?? '', [Validators.required, Validators.pattern(CustomRegExp.EMAIL)]),
        name: new UntypedFormControl('', [Validators.required, Validators.pattern(CustomRegExp.TEXT_WITHOUT_NUMBER)]),
        surname: new UntypedFormControl('', [
          Validators.required,
          Validators.pattern(CustomRegExp.TEXT_WITHOUT_NUMBER),
        ]),
        phone: new UntypedFormControl('', [Validators.required, Validators.pattern(CustomRegExp.PHONE)]),
        password: new UntypedFormControl('', [
          Validators.required,
          Validators.pattern(CustomRegExp.PASSWORD), // NOSONAR
        ]),
        confirmPassword: new UntypedFormControl(''),
        hasAcceptedNewsletter: new UntypedFormControl(null),
      },
      [MustMatch('password', 'confirmPassword')],
    );
  }

  private createProfileForm(): void {
    this.profileForm = new UntypedFormGroup({
      employer: new UntypedFormGroup({
        name: new UntypedFormControl('', [Validators.required]),
        validated: new UntypedFormControl(false, [Validators.required]),
      }),
      job: new UntypedFormGroup({
        name: new UntypedFormControl('', [Validators.required]),
        validated: new UntypedFormControl(true, [Validators.required]),
        hasPersonalOffer: new UntypedFormControl(true, [Validators.required]),
      }),
      withAppointment: new UntypedFormControl('', [Validators.required]),
      structure: new UntypedFormControl('', [Validators.required]),
    });
  }

  private createPersonalOfferForm(personalOffer: PersonalOffer): void {
    this.personalOfferForm = new UntypedFormGroup({
      categories: new UntypedFormGroup({
        onlineProcedures: new UntypedFormControl(personalOffer.categories.onlineProcedures),
        baseSkills: new UntypedFormControl(personalOffer.categories.baseSkills),
        advancedSkills: new UntypedFormControl(personalOffer.categories.advancedSkills),
      }),
    });
  }

  public validatePage(value = true): void {
    this.isPageValid = value;
    this.cdRef.detectChanges();
  }

  public nextPage(): void {
    this.isPageValid = false;
    // If the user is creating personalOffer for one of his structure from his profile, we skip the structure choice step
    if (
      this.isPersonalOfferProfile &&
      this.currentFormType === formType.personaloffer &&
      this.currentPage === personalOfferFormStep.personalOfferStructureChoice - 1
    ) {
      this.setHasOtherPersonalOffer(false);
      this.currentPage++;
    }
    if (this.currentPage < this.nbSteps) {
      this.currentPage++;
    }
  }
  public prevPage(): void {
    if (this.currentPage > 0) {
      this.currentPage--;
    }
  }

  public setHasOtherPersonalOffer(value: boolean): void {
    this.hasOtherPersonalOffer = value;
  }

  public linkStructureToUser(): void {
    this.structure._id = this.structureForm.value._id;
    this.structure.structureName = this.structureForm.value.structureName;

    if (this.structure.hasNoUserDN) {
      this.structureService.claimStructureWithAccount(this.structure._id, this.profile.email).subscribe(() => {
        this.currentPage = structureFormStep.mailSentInfo;
      });
    } else {
      this.usersService.joinStructure(this.structureForm.value._id, this.profile.email).subscribe(() => {
        this.currentPage = structureFormStep.mailSentInfo;
      });
    }
  }

  public async endForm(type: { formType: formType; formStep?: stepType }): Promise<void> {
    switch (type.formType) {
      case formType.account:
        break;
      case formType.profile:
        this.saveProfileForm();
        break;
      case formType.structure:
        if (type.formStep === structureFormStep.mailSentInfo) {
          const user = await this.profileService.getProfile();
          if (user.job?.hasPersonalOffer) {
            this.previousSteps = 0;
            this.router.navigateByUrl('formulaire/offre-personnelle');
          } else {
            this.router.navigateByUrl('/');
          }
        }
        if (type.formStep === structureFormStep.structureChoice) {
          // If structure already exist, join it.
          // On first strike the structure._id is not set and is a key to differentiate with other case
          if (this.structureForm.value._id && !this.structure._id) {
            this.linkStructureToUser();
          }
        }
        if (type.formStep === structureFormStep.structureCreationFinishedInfo) {
          // disable button to avoid multiple click that may create several identical structures
          this.isPageValid = false;
          this.saveStructureForm();
        }
        if (type.formStep === structureFormStep.noStructure) {
          this.router.navigateByUrl('/profil');
        }
        break;
      case formType.personaloffer:
        this.savePersonalOfferForm();
        break;
    }
  }

  public saveProfileForm(): void {
    forkJoin({
      employer: this.profileService.createEmployer(this.profileForm.get('employer').value).pipe(
        map((res) => res),
        catchError(() => of(this.profileForm.get('employer').value)),
      ),
      job: this.profileService.createJob(this.profileForm.get('job').value).pipe(
        map((res) => res),
        catchError(() => of(this.profileForm.get('job').value)),
      ),
      profile: this.profileService
        .updateProfile(
          this.profileForm.get('employer').value.name,
          this.profileForm.get('job').value.name,
          this.profileForm.get('withAppointment')?.value !== ''
            ? this.profileForm.get('withAppointment').value
            : undefined,
        )
        .pipe(
          map((res) => (this.profile = res)),
          catchError(() => of()),
        ),
    }).subscribe(async () => {
      // if register a new user as a new member structure, no structure to choose
      if (this.isRegisterNewMember) {
        if (this.profile.job?.hasPersonalOffer) {
          this.structureService.getStructure(this.profile.structuresLink[0]).subscribe((structure) => {
            this.structure = new Structure(structure);
            this.previousSteps = 0;
            this.router.navigateByUrl('formulaire/offre-personnelle');
          });
        } else {
          this.router.navigateByUrl('/profil');
        }
      } else {
        this.router.navigateByUrl('formulaire/structure');
      }
    });
  }

  public savePersonalOfferForm(): void {
    this.personalOfferService.createPersonalOffer(this.structure._id, this.personalOfferForm.value).subscribe(() => {
      if (this.hasOtherPersonalOffer) {
        this.router.navigateByUrl('formulaire/structure');
      } else {
        this.nextPage();
      }
    });
  }
  public async saveStructureForm(): Promise<void> {
    if (!this.utils.isOtherAccompanimentSelected(this.structureForm)) {
      this.structureForm.get('otherDescription').setValue(null);
    }
    const user = await this.profileService.getProfile();
    const newStructure = new Structure(this.structureForm.value);
    newStructure.hours = this.hoursForm.value;
    this.structureService.createStructure(newStructure, this.profile).subscribe((struct) => {
      if (user.job?.hasPersonalOffer) {
        this.structure = struct;
        this.previousSteps = 0;
        this.router.navigateByUrl('formulaire/offre-personnelle');
      } else {
        this.router.navigateByUrl(`acteurs?structure=${struct.permalink}`);
      }
    });
  }
  public setCurrentStep(step: stepType): void {
    // THIS PROBABLY CREATES CONSOLE ERRORS NG100 only in dev mode, please refer to https://angular.io/errors/NG0100 for more info
    this.isPageValid = false;
    this.currentPage = step;
  }

  public async saveEditedStructure(): Promise<void> {
    this.structureService.editStructure(this.getCurrentChanges(this.currentPage), this.structure._id).subscribe(() => {
      this.notificationService.showSuccess('Vos modifications ont bien été prises en compte.');
      history.back();
    });
  }

  /**
   * Returns only fields on the current step for edition.
   * We have to do this because some fields might not be completed and we cannot send the full object because of BO validation
   */
  public getCurrentChanges(currentPage: stepType): Partial<Structure> {
    switch (currentPage) {
      case structureFormStep.structureChoice:
        return {
          _id: this.structureForm.get('_id').value,
        };
      case structureFormStep.structureNameAndAddress:
        return {
          structureName: this.structureForm.get('structureName').value,
          address: this.structureForm.get('address').value,
        };
      case structureFormStep.structureType:
        return {
          structureType: this.structureForm.get('structureType').value,
        };
      case structureFormStep.structureContact:
        return {
          contactPhone: this.structureForm.get('contactPhone').value,
          contactMail: this.structureForm.get('contactMail').value,
        };
      case structureFormStep.structureAccessModality:
        return {
          categories: {
            accessModality: this.structureForm.get('categories').get('accessModality').value,
          },
        };
      case structureFormStep.structureHours:
        return {
          hours: this.hoursForm.value,
          exceptionalClosures: this.structureForm.get('exceptionalClosures').value,
        };
      case structureFormStep.structurePmr:
        return {
          pmrAccess: this.structureForm.get('pmrAccess').value,
        };
      case structureFormStep.structureWebAndSocialNetwork:
        return {
          facebook: this.structureForm.get('facebook').value,
          instagram: this.structureForm.get('instagram').value,
          linkedin: this.structureForm.get('linkedin').value,
          twitter: this.structureForm.get('twitter').value,
          website: this.structureForm.get('website').value,
        };
      case structureFormStep.structurePublicTarget:
        return {
          categories: {
            age: this.structureForm.get('categories').get('age').value,
          },
        };
      case structureFormStep.structurePublicTargetOptional:
        return {
          categories: {
            languageAndIlliteracy: this.structureForm.get('categories').get('languageAndIlliteracy').value,
            handicaps: this.structureForm.get('categories').get('handicaps').value,
            genre: this.structureForm.get('categories').get('genre').value,
          },
        };
      case structureFormStep.structureDigitalHelpingAccompaniment:
        return {
          categories: {
            onlineProcedures: this.structureForm.get('categories').get('onlineProcedures').value,
          },
        };
      case structureFormStep.structureDigitalHelpingAccompanimentOther:
        return {
          otherDescription: this.structureForm.get('otherDescription').value,
        };
      case structureFormStep.structureTrainingType:
        return {
          categories: {
            baseSkills: this.structureForm.get('categories').get('baseSkills').value,
            advancedSkills: this.structureForm.get('categories').get('advancedSkills').value,
          },
        };
      case structureFormStep.structureTrainingPrice:
        return {
          categories: {
            freeWorkShop: this.structureForm.get('categories').get('freeWorkShop').value,
          },
        };
      case structureFormStep.structureWifi:
        return {
          categories: {
            selfServiceMaterial: this.structureForm.get('categories').get('selfServiceMaterial').value,
          },
        };
      case structureFormStep.structureEquipments:
        return {
          nbComputers: this.structureForm.get('nbComputers').value,
          nbPrinters: this.structureForm.get('nbPrinters').value,
          nbScanners: this.structureForm.get('nbScanners').value,
        };
      case structureFormStep.structureLabels:
        return {
          categories: {
            labelsQualifications: this.structureForm.get('categories').get('labelsQualifications').value,
          },
        };
      case structureFormStep.structureSolidarityMaterial:
        return {
          categories: {
            solidarityMaterial: this.structureForm.get('categories').get('solidarityMaterial').value,
          },
        };
      case structureFormStep.structureDescription:
        return {
          description: this.structureForm.get('description').value,
        };
      case structureFormStep.structureConsent:
        return {
          dataShareConsentDate: this.structureForm.get('dataShareConsentDate').value ? new Date().toString() : null,
        };
    }
  }

  private setExitPages(): number[] {
    switch (this.currentFormType) {
      case formType.structure:
        return [
          structureFormStep.noStructure,
          structureFormStep.structureCreationFinishedInfo,
          structureFormStep.mailSentInfo,
        ];
      case formType.profile:
        return [profileFormStep.profileJobSelection];
      case formType.personaloffer:
        return [personalOfferFormStep.personalOfferFinishedInfo];
      default:
        return [];
    }
  }

  public canExit(): Promise<boolean> {
    // List all exit pages in order to authorize exit
    const exitPages: stepType[] = this.setExitPages();
    // Avoid confirmation when user submit form and leave.
    if (
      this.currentPage === this.nbSteps ||
      this.currentPage < 1 ||
      this.isEditMode ||
      exitPages.includes(this.currentPage)
    ) {
      return Promise.resolve(true);
    } else {
      return new Promise((resolve) => this.showModal(resolve));
    }
  }
  private showModal(resolve: CanExitResolver): void {
    this.showConfirmationModal = true;
    this.resolve = resolve;
  }
  public hasRedirectionAccepted(hasAccept: boolean): void {
    this.resolve(hasAccept);
    this.showConfirmationModal = false;
  }
}
