import { Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges } from '@angular/core';
import { AbstractControl, UntypedFormArray, UntypedFormControl, UntypedFormGroup } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { CategoriesToggle } from 'src/app/models/categoriesToggle.model';
import { Address } from '../../../models/address.model';
import { Structure } from '../../../models/structure.model';
import { User } from '../../../models/user.model';
import { ProfileService } from '../../../profile/services/profile.service';
import { CategoryEnum } from '../../../shared/enum/category.enum';
import { Category } from '../../../structure-list/models/category.model';
import { Module } from '../../../structure-list/models/module.model';
import { SearchService } from '../../../structure-list/services/search.service';
import { stepType } from '../../step.type';
import { formType } from '../formType.enum';
import { structureFormStep } from './structureFormStep.enum';

@Component({
  selector: 'app-structure-form',
  templateUrl: './structure-form.component.html',
})
export class StructureFormComponent implements OnInit, OnChanges {
  @Input() nbSteps: number;
  @Input() currentStep: stepType;
  @Input() structure: Structure;
  @Input() structureForm: UntypedFormGroup;
  @Input() hoursForm: UntypedFormGroup;
  @Input() isEditMode: boolean;
  @Output() pageValid = new EventEmitter<any>();
  @Output() updateHoursForm = new EventEmitter<any>();
  @Output() goNext = new EventEmitter<any>();
  @Output() setEditStep = new EventEmitter<any>();
  public structureFormStep = structureFormStep;
  public formTypeEnum = formType;
  public isPageValid: boolean;
  public pagesValidation = [];
  public profile: User;

  // Condition form
  public isAccountMode = false;
  public userAcceptSavedData = false;

  // Form var
  public equipments: { module: Module; openned: boolean }[] = [];
  public trainingCategories: CategoriesToggle[] = [];
  public accessModality: Category;
  public labelsQualifications: Category;
  public onlineProcedures: Category;
  public age: Category;
  public genre: Category;
  public handicaps: Category;
  public languageAndIlliteracy: Category;
  public solidarityMaterial: Category;

  constructor(
    private searchService: SearchService,
    private profileService: ProfileService,
    private route: ActivatedRoute,
  ) {}

  ngOnInit(): void {
    this.setCategories();
    this.profileService.getProfile().then((user) => {
      this.profile = user;
    });
    if (this.isEditMode) {
      this.route.params.subscribe((params) => {
        this.setEditStep.emit(structureFormStep[params.step]);
      });
    }
    this.setValidationsForm();
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.currentStep && this.pagesValidation.length > 0) {
      this.updatePageValid();
    }
  }

  public setAddressStructure(address?: Address): void {
    if (address) {
      this.structureForm.get('address').get('numero').setValue(address.numero);
      this.structureForm.get('address').get('street').setValue(address.street);
      this.structureForm.get('address').get('commune').setValue(address.commune);
    } else {
      this.structureForm.get('address').reset();
    }
  }

  public setTypeStructure(type?: string): void {
    this.structureForm.get('structureType').setValue(type);
  }

  public updateHours(form: UntypedFormGroup): void {
    this.hoursForm = form;
    this.updateHoursForm.emit(form);
  }

  public setValidationsForm(): void {
    this.pagesValidation[structureFormStep.structureChoice] = {
      valid: this.structureForm.get('_id').valid,
    };
    this.pagesValidation[structureFormStep.structureNameAndAddress] = {
      valid: this.structureForm.get('structureName').valid && this.structureForm.get('address').valid,
    };
    this.pagesValidation[structureFormStep.structureContact] = {
      valid:
        !this.structureForm.hasError('requirePhoneOrMail') &&
        this.structureForm.get('contactMail').valid &&
        this.structureForm.get('contactPhone').valid,
    };
    this.pagesValidation[structureFormStep.structureAccompanimentChoice] = {
      valid: this.structureForm.get('placeOfReception').valid,
    };
    this.pagesValidation[structureFormStep.structureType] = {
      valid: this.structureForm.get('structureType').valid,
    };
    this.pagesValidation[structureFormStep.structureAccessModality] = {
      valid: this.structureForm.get('categories').get('accessModality').valid,
    };
    this.pagesValidation[structureFormStep.structureHours] = {
      valid: this.hoursForm.valid && this.structureForm.get('exceptionalClosures').valid,
    };
    this.pagesValidation[structureFormStep.structurePmr] = {
      valid: this.structureForm.get('pmrAccess').valid,
    };
    this.pagesValidation[structureFormStep.structureWebAndSocialNetwork] = {
      valid:
        this.structureForm.get('website').valid &&
        this.structureForm.get('facebook').valid &&
        this.structureForm.get('twitter').valid &&
        this.structureForm.get('instagram').valid,
    };
    this.pagesValidation[structureFormStep.structurePublicTarget] = {
      valid: this.structureForm.get('categories').get('age').valid,
    };
    this.pagesValidation[structureFormStep.structureDigitalHelpingAccompaniment] = {
      valid: this.structureForm.get('categories').get('onlineProcedures').valid,
    };
    this.pagesValidation[structureFormStep.structureTrainingPrice] = {
      valid: this.structureForm.get('categories').get('freeWorkShop').valid,
    };
    this.pagesValidation[structureFormStep.structureWifi] = {
      valid: this.structureForm.get('freeWifi').valid,
    };
    this.pagesValidation[structureFormStep.structureEquipments] = {
      valid:
        this.structureForm.get('nbComputers').valid &&
        this.structureForm.get('nbPrinters').valid &&
        this.structureForm.get('nbScanners').valid,
      name: 'Equipements mis à disposition',
    };
    this.pagesValidation[structureFormStep.structureConsent] = {
      valid: !this.isEditMode ? this.userAcceptSavedData : true,
    };

    const optionalSteps = [
      structureFormStep.structureDescription,
      structureFormStep.structureCreationFinishedInfo,
      structureFormStep.structureTrainingType,
      structureFormStep.structurePublicTargetOptional,
      structureFormStep.structureDigitalHelpingAccompanimentOther,
      structureFormStep.structureEquipments,
      structureFormStep.structureLabels,
      structureFormStep.structureSolidarityMaterial,
      structureFormStep.structureDescription,
      structureFormStep.mailSentInfo,
      structureFormStep.structureFormTime,
      structureFormStep.noStructure,
    ];
    for (const step of optionalSteps) {
      this.pagesValidation[step] = { valid: true };
    }

    this.updatePageValid();
  }

  /**
   * Update valid page or return page validity of the given index
   */
  private updatePageValid(): void {
    this.isPageValid = this.pagesValidation[this.currentStep].valid;
    this.pageValid.emit(this.isPageValid);
  }

  async setCategories(): Promise<void> {
    const categories = await this.searchService.getCategories().toPromise();
    categories.forEach((categ) => {
      switch (categ.id) {
        case CategoryEnum.onlineProcedures: {
          // We sort modules alphabetically, except for the 'autres' module which is always at the end
          this.onlineProcedures = categ;
          const othersModule = this.onlineProcedures.modules.find((module) => module.id === 'autres');
          const sortedModules = this.onlineProcedures.modules
            .filter((module) => module.id !== 'autres')
            .sort((a, b) => a.name.localeCompare(b.name));
          if (othersModule) sortedModules.push(othersModule);
          this.onlineProcedures.modules = sortedModules;
          break;
        }
        case CategoryEnum.baseSkills:
        case CategoryEnum.advancedSkills: {
          this.trainingCategories.push({ category: categ, openned: false });
          break;
        }
        case CategoryEnum.accessModality: {
          this.accessModality = categ;
          break;
        }
        case CategoryEnum.selfServiceMaterial: {
          categ.modules.forEach((c) => {
            this.equipments.push({ module: c, openned: false });
          });
          break;
        }
        case CategoryEnum.labelsQualifications: {
          this.labelsQualifications = categ;
          break;
        }
        case CategoryEnum.age: {
          this.age = categ;
          break;
        }
        case CategoryEnum.languageAndIlliteracy: {
          this.languageAndIlliteracy = categ;
          break;
        }
        case CategoryEnum.genre: {
          this.genre = categ;
          break;
        }
        case CategoryEnum.solidarityMaterial: {
          this.solidarityMaterial = categ;
          break;
        }
        case CategoryEnum.handicaps: {
          this.handicaps = categ;
          break;
        }
      }
    });
  }

  public onCheckChange({
    event,
    formControlName,
    value,
  }: {
    event: boolean;
    formControlName: string;
    value: string;
  }): void {
    const formArray: UntypedFormArray = this.structureForm.get(formControlName) as UntypedFormArray;
    if (event) {
      // Add a new control in the arrayForm
      formArray.push(new UntypedFormControl(value));
    } else {
      // Remove uncheck control in the arrayForm
      const index = formArray.controls.findIndex((element) => element.value === value);
      formArray.removeAt(index);
    }
    this.setValidationsForm();
  }

  public updateChoice({ formControlName, choice }: { formControlName: string; choice: string }): void {
    const event = !this.isInArray({ formControlName, term: choice });
    this.onCheckChange({ event, formControlName, value: choice });
  }

  // Check if a FormControl value is in FormArray
  public isInArray({ formControlName, term }: { formControlName: string; term: string }): boolean {
    if (this.structureForm.get(formControlName)?.value) {
      return this.structureForm.get(formControlName).value.includes(term);
    }
    return false;
  }

  public onRadioChange({ name, value }: { name: string; value: string | boolean }): void {
    if (name === 'freeWorkShop') {
      this.structureForm.get('categories').get(name).setValue(value);
    } else {
      this.structureForm.get(name).setValue(value);
    }
  }

  public getStructureControl(nameControl: string): AbstractControl {
    return this.structureForm.get(nameControl);
  }

  public acceptDataBeSaved(isAccepted: boolean): void {
    this.userAcceptSavedData = isAccepted;
  }

  public acceptOpenData(isAccepted: boolean): void {
    const now = isAccepted ? new Date().toString() : '';
    this.getStructureControl('dataShareConsentDate').setValue(now);
  }

  public goToCreateStructure(structureName = ''): void {
    this.structureForm.patchValue({ _id: null, structureName });
    this.goNext.emit();
  }
}
