import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { Observable, forkJoin } from 'rxjs';
import { map, tap } from 'rxjs/operators';
import { Owner } from '../../../../../models/owner.model';
import { Structure } from '../../../../../models/structure.model';
import { OrientationService } from '../../../../../services/orientation.service';
import { StructureService } from '../../../../../services/structure.service';

@Component({
  selector: 'app-make-appointment',
  templateUrl: './make-appointment.component.html',
  styleUrls: ['./make-appointment.component.scss'],
})
export class MakeAppointmentComponent implements OnInit {
  @Input() form: UntypedFormGroup;
  @Input() structureRDV: Structure;
  @Output() checkValidation = new EventEmitter<any>();
  @Output() socialWorker = new EventEmitter<Owner>();
  @Output() failedOrientation = new EventEmitter<void>();
  @Output() selectedStructureRDV = new EventEmitter<Structure>();

  public owners: Owner[];
  public structures: Structure[] = [];
  public structuresListReady: boolean;
  public comingFromRegistry: boolean;

  private onlineDemarcheType;
  private filtersOnOffers;

  constructor(
    private structureService: StructureService,
    public orientationService: OrientationService,
  ) {}

  ngOnInit(): void {
    this.checkValidation.emit();
    this.comingFromRegistry = this.orientationService.rdvUser !== null;

    // When no appointment available, we need to know the name of the filters requested to display them
    if (this.structureRDV?._id || this.orientationService.rdvUser) {
      this.onlineDemarcheType = this.form.get('onlineDemarcheType').value;
      this.filtersOnOffers = { onlineProcedures: [], baseSkills: [] };
      this.onlineDemarcheType.forEach((filter) => {
        if (filter.name === 'onlineProcedures') {
          this.filtersOnOffers.onlineProcedures.push(filter.value);
        }
        if (filter.name === 'baseSkills') {
          this.filtersOnOffers.baseSkills.push(filter.value);
        }
      });
    }

    // Filters owners that accept appointments, have personal offers in the selected structure and match the needs previously selected
    if (!this.comingFromRegistry) {
      this.findOwners(this.structureRDV._id).subscribe((owners) => {
        const filteredOwners = [];
        this.owners = owners.filter((owner) => owner.withAppointment && owner.job?.hasPersonalOffer);
        // make an array with the IDs of the structure's personalOffers
        const structurePersonalOffersId = this.structureRDV.personalOffers.map((spo) => spo._id);
        // for each owner
        this.owners.forEach((owner) => {
          // check each of its personnalOffers
          owner.personalOffers?.forEach((po) => {
            // if this personalOffer matches one of the structure's personalOffers
            if (structurePersonalOffersId.includes(po._id)) {
              // and if the needs the user selected are all part of the personalOffer, keep the owner for display
              if (
                this.filtersOnOffers.onlineProcedures.every((offer) =>
                  po.categories.onlineProcedures.includes(offer),
                ) &&
                this.filtersOnOffers.baseSkills.every((offer) => po.categories.baseSkills.includes(offer))
              ) {
                if (!filteredOwners.includes(owner)) {
                  filteredOwners.push(owner);
                }
              }
            }
          });
        });

        this.owners = filteredOwners;

        if (this.owners.length === 0) {
          this.failedOrientation.emit();
        }
      });
    } else {
      // If we come from the registry, list the registry's user's structures
      const structures$: Observable<any>[] = [];
      this.structures = [];
      this.orientationService.rdvUser.structuresLink.forEach((structureId) => {
        structures$.push(
          this.structureService.getStructure(structureId).pipe(
            tap((structure) => {
              this.structures.push(new Structure(structure));
            }),
          ),
        );
      });
      forkJoin(structures$).subscribe(() => {
        this.structures.sort((a, b) => a.structureName.localeCompare(b.structureName));

        // filter structures that match selected needs
        const filteredStructures = [];
        // for each structure
        this.structures.forEach((structure) => {
          // check each of its personnalOffers
          structure.personalOffers?.forEach((po) => {
            // if the needs the user selected are all part of the personalOffer, keep the structure for display
            if (
              this.filtersOnOffers.onlineProcedures.every((offer) => po.categories.onlineProcedures.includes(offer)) &&
              this.filtersOnOffers.baseSkills.every((offer) => po.categories.baseSkills.includes(offer))
            ) {
              filteredStructures.push(structure);
            }
          });
        });

        this.structures = filteredStructures;

        if (this.structures.length === 0) {
          this.failedOrientation.emit();
        }
        this.structuresListReady = true;
      });
    }
  }

  private findOwners(structureId: string): Observable<Owner[]> {
    return this.structureService.getStructureWithOwners(structureId).pipe(map((result) => result.owners));
  }

  public onSocialWorkerRadioChange(socialWorker: Owner): void {
    const structure = this.structureRDV ?? this.orientationService.rdvStructure;
    if (this.structureRDV)
      this.form.patchValue({
        structureRDV: structure._id,
        socialWorkerId: socialWorker._id,
      });
    this.socialWorker.emit(socialWorker);
    this.checkValidation.emit();
  }

  public onStructureRadioChange(structure: Structure): void {
    this.form.patchValue({
      structureRDV: structure._id,
      socialWorkerId: this.orientationService.rdvUser._id,
    });
    this.selectedStructureRDV.emit(structure);
    this.checkValidation.emit();
  }

  public getFormattedNeeds(): string {
    return this.form
      .get('onlineDemarcheType')
      ?.value.map((n) => n.text)
      .join(', ');
  }
}
