import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { FormGroup, UntypedFormArray, UntypedFormControl, UntypedFormGroup } from '@angular/forms';
import { Owner } from '../../../models/owner.model';
import { Structure } from '../../../models/structure.model';
import { User } from '../../../models/user.model';
import { OrientationService } from '../../../services/orientation.service';
import { Category } from '../../../structure-list/models/category.model';
import { Filter } from '../../../structure-list/models/filter.model';
import { OrientationUtils } from '../../../utils/orientationUtils';
import {
  AccompanimentTypes,
  AppointmentSteps,
  CommonSteps,
  GenericOrientationSteps,
  NeedsTypes,
  OnlineMediationSteps,
  StructuresListSteps,
} from '../enums/orientation.enums';
import { FiltersForm } from '../interfaces/filtersForm.interface';
import { AllOrientationSteps } from '../types/orientation.types';

@Component({
  selector: 'app-online-demarch-form',
  templateUrl: './onlineDemarch-form.component.html',
})
export class OnlineDemarchFormComponent implements OnInit {
  @Input() needType: NeedsTypes;
  @Input() currentStep: AllOrientationSteps;
  @Input() genericStep: GenericOrientationSteps;
  @Input() accompanimentType: AccompanimentTypes;
  @Input() form: UntypedFormGroup;
  @Input() filters: Filter[] = [];
  @Input() filtersForm: FormGroup<FiltersForm>;
  @Input() profile: User;
  @Input() categories: Record<string, Category>;
  @Output() validatePage = new EventEmitter<any>();
  @Output() validateStructureRDV = new EventEmitter<Structure>();
  @Output() failedOrientation = new EventEmitter<any>();

  public orientationUtils = new OrientationUtils();
  public pagesValidation: any[] = [];
  public structureRDV: Structure;
  public socialWorker: Owner;

  // Enums
  public GenericOrientationSteps = GenericOrientationSteps;
  public NeedsTypes = NeedsTypes;
  public AccompanimentTypes = AccompanimentTypes;
  public CommonSteps = CommonSteps;
  public StructuresListSteps = StructuresListSteps;
  public OnlineDemarchesAppointmentSteps = AppointmentSteps;
  public OnlineMediationSteps = OnlineMediationSteps;

  constructor(public orientationService: OrientationService) {}

  ngOnInit(): void {
    // If coming from registry
    if (this.orientationService.rdvUser) {
      this.setSocialWorker(this.orientationService.rdvUser as unknown as Owner);
    }

    // If coming from carto
    if (this.orientationService.rdvStructure) {
      this.setStructureRDV(this.orientationService.rdvStructure);
    }
  }

  public checkValidation(event?: any): void {
    if (this.genericStep === GenericOrientationSteps.common) {
      this.orientationUtils.setValidationsOnlineDemarchesForm(
        this.pagesValidation,
        this.filtersForm,
        this.form,
        (isValid) => this.validatePage.emit(isValid),
        this.currentStep as CommonSteps,
      );
      return;
    }

    switch (this.accompanimentType) {
      case AccompanimentTypes.structuresList:
        this.orientationUtils.setValidationsStructuresForm(
          this.pagesValidation,
          this.form,
          (isValid) => this.validatePage.emit(isValid),
          this.currentStep as StructuresListSteps,
        );
        break;
      case AccompanimentTypes.appointment:
        if (this.currentStep === AppointmentSteps.carto - 1) {
          // if rdv from structure details, don't reinit the structureRDV variable
          if (!this.orientationService.rdvStructure) this.structureRDV = null;
        }
        if (this.currentStep === AppointmentSteps.carto) {
          this.structureRDV = event;
          this.validateStructureRDV.emit(event);
          return;
        }
        this.orientationUtils.setValidationsAppointmentForm(
          this.pagesValidation,
          this.form,
          (isValid) => this.validatePage.emit(isValid),
          this.currentStep as AppointmentSteps,
        );
        break;
      case AccompanimentTypes.onlineMediation:
        this.orientationUtils.setValidationsOnlineMediationForm(
          this.pagesValidation,
          this.form,
          (isValid) => this.validatePage.emit(isValid),
          this.currentStep as OnlineMediationSteps,
        );
        break;
      default:
        throw new Error('Not implemented tunnel type in OnlineDemarchFormComponent');
    }
  }

  public setSocialWorker(item: Owner): void {
    this.socialWorker = item;
  }
  public setStructureRDV(structure: Structure): void {
    this.structureRDV = structure;
  }

  public updatePublicChoice({ formControlName, choice }: { formControlName: string; choice: string }): void {
    const event = !this.isInArray({ formControlName, term: choice });
    this.onCheckChange({ event, formControlName, value: choice });
  }

  public onCheckChange({
    event,
    formControlName,
    value,
  }: {
    event: boolean;
    formControlName: string;
    value: string;
  }): void {
    const formArray: UntypedFormArray = this.form.get(formControlName) as UntypedFormArray;
    if (event) {
      // Add a new control in the arrayForm
      formArray.push(new UntypedFormControl(value));
    } else {
      // Remove uncheck control in the arrayForm
      const index = formArray.value.findIndex((element) => element === value);
      formArray.removeAt(index);
    }
    this.checkValidation();
  }
  // Check if a FormControl value is in FormArray
  public isInArray({ formControlName, term }: { formControlName: string; term: string }): boolean {
    if (this.form.get(formControlName)?.value) {
      return this.form.get(formControlName).value.includes(term);
    }
    return false;
  }
  public onRadioChange(event: { name: string; value: boolean }): void {
    const { name, value } = event;
    this.form.get(name).setValue(value);
    this.orientationUtils.manuallySetOfPmr(this.filters, event);
    this.checkValidation();
  }
  public setFailedOrientation(): void {
    this.failedOrientation.emit();
  }
}
