import { AfterContentChecked, ChangeDetectorRef, Component, OnInit, ViewChild } from '@angular/core';
import { AbstractControl, FormGroup, UntypedFormGroup } from '@angular/forms';
import { Router } from '@angular/router';
import { lastValueFrom } from 'rxjs';
import { Structure } from '../../models/structure.model';
import { User } from '../../models/user.model';
import { ProfileService } from '../../profile/services/profile.service';
import { AuthService } from '../../services/auth.service';
import { IndicatorService } from '../../services/indicator.service';
import { NotificationService } from '../../services/notification.service';
import { OrientationService } from '../../services/orientation.service';
import { StructureService } from '../../services/structure.service';
import { CategoryEnum } from '../../shared/enum/category.enum';
import { Category } from '../../structure-list/models/category.model';
import { Filter } from '../../structure-list/models/filter.model';
import { Module } from '../../structure-list/models/module.model';
import { SearchService } from '../../structure-list/services/search.service';
import { OrientationUtils } from '../../utils/orientationUtils';
import { CanExitResolver, Utils } from '../../utils/utils';
import { formType } from '../form-view/formType.enum';
import {
  AccompanimentTypes,
  AppointmentSteps,
  CommonSteps,
  GenericOrientationSteps,
  NeedsTypes,
  OnlineMediationSteps,
  StructuresListSteps,
} from './enums/orientation.enums';
import { NavigationComponent } from './global-components/navigation/navigation.component';
import { FiltersForm } from './interfaces/filtersForm.interface';
import {
  IOrientationIndicator,
  OrientationIndicatorStructure,
  OrientationIndicatorType,
} from './interfaces/orientationIndicator.interface';
import { StructureOrientationForm } from './interfaces/structureOrientationForm.interface';
import { StructureOrientator } from './interfaces/structureOrientator.interface';
import { AllOrientationSteps } from './types/orientation.types';

@Component({
  selector: 'app-orientation-form-view',
  templateUrl: './orientation-form-view.component.html',
  styleUrls: ['./orientation-form-view.component.scss'],
})
export class OrientationFormViewComponent implements OnInit, AfterContentChecked {
  public static readonly MAX_STEP =
    (Math.max(
      Object.keys(StructuresListSteps).length,
      Object.keys(AppointmentSteps).length,
      Object.keys(OnlineMediationSteps).length,
    ) +
      Object.keys(CommonSteps).length) /
    2;
  public orientationUtils = new OrientationUtils();
  public utils = new Utils();
  // OnlineDemarches
  public onlineDemarcheForm: UntypedFormGroup;
  // Equipments
  public filtersForm: FormGroup<FiltersForm>;
  public structureOrientationForm: FormGroup<StructureOrientationForm>;
  // Orientator
  public orientator: FormGroup<StructureOrientator>;
  // Enum
  public NeedsTypes = NeedsTypes;
  public formType = formType;
  // Global
  public needType: NeedsTypes;
  public genericStep: GenericOrientationSteps = GenericOrientationSteps.common;
  public accompanimentType: AccompanimentTypes;
  public currentStep: AllOrientationSteps = null;
  public isPrevHidden = false;
  public isNextHidden = false;
  public isLastStep = false;
  public nbSteps = OrientationFormViewComponent.MAX_STEP;
  public isPageValid = false;
  public filters: Filter[] = [];
  public profile: User;
  public canDeactivate = false;
  public showLoginModal = false;
  public fullScreen = false;
  public failedOrientation = false;
  // Categories
  public categories: Record<string, Category> = {
    age: {
      id: '',
      name: '',
      theme: '',
      modules: [],
    },
    languageAndIlliteracy: {
      id: '',
      name: '',
      theme: '',
      modules: [],
    },
    genre: {
      id: '',
      name: '',
      theme: '',
      modules: [],
    },
    handicaps: {
      id: '',
      name: '',
      theme: '',
      modules: [],
    },
  };

  // Modal canExit var
  public showConfirmationModal = false;
  private resolve: CanExitResolver;

  @ViewChild(NavigationComponent) navComponent!: NavigationComponent;

  constructor(
    public orientationService: OrientationService,
    private notificationService: NotificationService,
    private authService: AuthService,
    private profilService: ProfileService,
    private profileService: ProfileService,
    private searchService: SearchService,
    private structureService: StructureService,
    private cdRef: ChangeDetectorRef,
    private indicatorService: IndicatorService,
    private router: Router,
  ) {
    this.setCategories();
  }

  async ngOnInit(): Promise<void> {
    this.orientationService.rdvUser = null;
    if (history.state.rdvUser) {
      this.orientationService.rdvUser = new User(history.state.rdvUser);
    }

    this.orientationService.rdvStructure = null;
    if (history.state.rdvStructure) {
      // Create new Structure object to call Structure functions (getTypeStructureIcon, ...)
      this.orientationService.rdvStructure = new Structure(history.state.rdvStructure);
    }

    if (!this.authService.isLoggedIn()) {
      this.showLoginModal = true;
      return;
    }

    this.profile = await this.profileService.getProfile();

    if (this.profile?.structuresLink?.length === 1) {
      this.structureService.getStructure(this.profile.structuresLink[0]).subscribe(
        (_structure) =>
          (this.orientator = this.orientationUtils.createStructureOrientatorForm({
            structureName: _structure.structureName,
            structureMail: _structure.contactMail,
            structurePhone: _structure.contactPhone,
          })),
      );
    }
  }

  ngAfterContentChecked(): void {
    this.cdRef.detectChanges();
  }

  public validatePage(event: boolean): void {
    this.isPageValid = event;
  }

  public setFailedOrientation(): void {
    this.failedOrientation = true;
  }

  public validateStructureRDV(event: Structure): void {
    this.onlineDemarcheForm.get('structureRDV').patchValue(event._id);
    this.nextPage();
  }

  private async handleAppointment(): Promise<void> {
    switch (this.accompanimentType) {
      case AccompanimentTypes.appointment:
        await lastValueFrom(
          this.orientationService.createAppointment({
            ...this.onlineDemarcheForm.value,
            onlineDemarcheType: this.onlineDemarcheForm.value.onlineDemarcheType.filter(
              (item) => item.name !== 'pmrAccess',
            ),
          }),
        );
        break;
      case AccompanimentTypes.onlineMediation:
        await lastValueFrom(
          this.orientationService.createOnlineMediation({
            ...this.onlineDemarcheForm.value,
            onlineDemarcheType: this.onlineDemarcheForm.value.filters.map((module: Module) => module.displayText),
          }),
        );
        break;
    }
  }

  private async printForm(): Promise<void> {
    await this.sendOrientationIndicator(this.structureOrientationForm ?? this.onlineDemarcheForm);
    window.print();
    setTimeout(() => {
      history.pushState({ rdvStructure: null, rdvUser: null }, '', '/orientation');
      window.location.reload();
    }, 100);
  }

  private async setCategories(): Promise<void> {
    const categories = await this.searchService.getCategories().toPromise();
    categories.forEach((category) => {
      switch (category.id) {
        case CategoryEnum.age: {
          this.categories.age = category;
          break;
        }
        case CategoryEnum.languageAndIlliteracy: {
          this.categories.languageAndIlliteracy = category;
          break;
        }
        case CategoryEnum.genre: {
          this.categories.genre = category;
          break;
        }
        case CategoryEnum.handicaps: {
          this.categories.handicaps = category;
          break;
        }
      }
    });
  }

  public async nextPage(): Promise<void> {
    this.isPageValid = false;
    this.isNextHidden = false;

    // Go next routine
    this.goToNextStep();

    // Check if the new screen must be skipped
    this.checkSkipInfoScreen(true);
    this.checkHasEligibleStructures();
    this.checkSkipCartoScreens('next');
    this.checkSkipStructureOrientator(true);

    // Handle appointments
    if (this.shouldHandleAppointment()) {
      await this.handleAppointment()
        .then(() => {
          this.notificationService.showSuccess('Votre demande a bien été enregistrée');
          this.canDeactivate = true;
        })
        .catch(() => {
          this.notificationService.showErrorPleaseRetry('Échec de la création de votre rendez-vous');
          this.goToPreviousStep();
        });
    }

    // Additional params (fullScreen, Nav buttons, ..)
    this.updateHiddenNavButtons();
    this.updateFullScreen();
    this.updateLastStep();
  }

  public prevPage(): void {
    // Go back routine
    this.goToPreviousStep();

    // Check if the new screen must be skipped
    this.checkSkipStructureOrientator(false);
    this.checkSkipCartoScreens('prev');
    this.checkSkipInfoScreen(false);

    // Additional params (fullScreen, Nav buttons, ..)
    this.updateHiddenNavButtons();
    this.updateFullScreen();
    this.updateLastStep();
  }

  private goToNextStep(): void {
    // Form starts
    if (this.currentStep === null) {
      this.currentStep = 0;
      this.filtersForm = this.orientationUtils.createFiltersForm();
      if (this.isOnlineDemarchOrBaseSkills()) {
        this.onlineDemarcheForm = this.orientationUtils.createOnlineDemarchesForm();
      }
      return;
    }

    // Deals with last steps
    if (
      this.tunnelingStepIs(AccompanimentTypes.structuresList, StructuresListSteps.orientationRecap) ||
      this.tunnelingStepIs(AccompanimentTypes.appointment, AppointmentSteps.orientationPrint) ||
      this.tunnelingStepIs(AccompanimentTypes.onlineMediation, OnlineMediationSteps.orientationPrint)
    ) {
      this.printForm();
      return;
    }

    // End of common steps, need to enter the accompaniment tunnels
    const commonEquipmentFinished = !this.isOnlineDemarchOrBaseSkills() && this.commonStepIs(CommonSteps.filtersChoice);
    const commonOnlineFinished = this.isOnlineDemarchOrBaseSkills() && this.commonStepIs(CommonSteps.accompanimentType);
    const commonAppointmentFinished =
      this.isComingFromCartoOrRegistry() && this.commonStepIs(CommonSteps.filtersChoice);

    if (commonEquipmentFinished || commonOnlineFinished || commonAppointmentFinished) {
      this.filters = this.filtersForm.value.filters.map((module: Module) => {
        return new Filter(this.needType, module.id, module.displayText);
      });
      // For skills orientation, only show by default structures with free or underCondition workshops
      if (this.needType === NeedsTypes.baseSkills) {
        this.filters.push(new Filter('freeWorkShop', 'yes', 'Accompagnements gratuits'));
        this.filters.push(new Filter('freeWorkShop', 'underCondition', 'Accompagnements gratuits sous conditions'));
      }
      this.startsAccompanimentTunnel();
      return;
    }

    this.currentStep++;
  }

  private goToPreviousStep(): void {
    if (this.currentStep === 0) {
      switch (this.genericStep) {
        // If back to beginning of the form, reset
        case GenericOrientationSteps.common:
          this.reset();
          break;

        // Else, we are in middle of form, going back to choice of accompaniment type
        case GenericOrientationSteps.accompanimentTunnel:
          this.genericStep = GenericOrientationSteps.common;
          this.currentStep = CommonSteps.filtersChoice;
          if (this.isOnlineDemarchOrBaseSkills()) {
            this.onlineDemarcheForm = this.orientationUtils.createOnlineDemarchesForm();
            if (!this.isComingFromCartoOrRegistry()) {
              this.currentStep = CommonSteps.accompanimentType;
            }
          }
          break;
      }
      return;
    }

    this.currentStep--;
  }

  private startsAccompanimentTunnel(): void {
    this.genericStep = GenericOrientationSteps.accompanimentTunnel;
    if (this.isComingFromCartoOrRegistry()) {
      this.accompanimentType = AccompanimentTypes.appointment;
      this.onlineDemarcheForm.get('accompanimentType').patchValue(AccompanimentTypes.appointment);
    } else if (!this.isOnlineDemarchOrBaseSkills()) {
      this.accompanimentType = AccompanimentTypes.structuresList;
    } else {
      const onlineProcedures: Filter[] = this.onlineDemarcheForm.value.onlineDemarcheType.map((module: Module) => {
        return new Filter('onlineProcedures', module.id, module.displayText);
      });
      if (onlineProcedures.length > 0) this.filters = [...onlineProcedures, ...this.filters];
      this.accompanimentType = this.onlineDemarcheForm.get('accompanimentType').value;
    }

    switch (this.accompanimentType) {
      case AccompanimentTypes.structuresList:
        this.currentStep = StructuresListSteps.pmrAccess;
        this.setStepNumber(Object.keys(StructuresListSteps).length);
        if (this.isOnlineDemarchOrBaseSkills()) {
          this.onlineDemarcheForm = this.orientationUtils.createStructuresListForm(
            this.utils.convertFiltersToModule(this.filters),
          );
        } else {
          this.structureOrientationForm = this.orientationUtils.createStructuresListForm(
            this.filtersForm.value.filters,
            this.orientator,
          );
        }
        break;
      case AccompanimentTypes.appointment:
        this.currentStep = AppointmentSteps.infoScreen;
        this.setStepNumber(Object.keys(AppointmentSteps).length);
        this.onlineDemarcheForm = this.orientationUtils.createAppointmentForm(this.filters, this.orientator);
        break;
      case AccompanimentTypes.onlineMediation:
        this.currentStep = OnlineMediationSteps.infoScreen;
        this.setStepNumber(Object.keys(OnlineMediationSteps).length);
        this.onlineDemarcheForm = this.orientationUtils.createOnlineMediationForm(
          this.utils.convertFiltersToModule(this.filters),
          this.orientator,
        );
        break;
      default:
        throw new Error('Unknown tunnel routine');
    }
  }

  private async checkHasEligibleStructures(): Promise<void> {
    if (this.tunnelingStepIs(AccompanimentTypes.appointment, AppointmentSteps.location)) {
      const eligibleStructures = (
        await lastValueFrom(this.structureService.getStructures(this.filters, 'search', true))
      ).filter((structure) => structure.hasUserWithAppointmentDN);
      if (eligibleStructures.length === 0) {
        this.sendOrientationIndicator(this.structureOrientationForm ?? this.onlineDemarcheForm);
        this.failedOrientation = true;
        this.currentStep = AppointmentSteps.rdvEnd;
      }
    }
  }

  public reset(): void {
    this.onlineDemarcheForm = undefined;
    this.filtersForm = this.orientationUtils.createFiltersForm();
    this.currentStep = null;
    this.genericStep = GenericOrientationSteps.common;
    this.needType = undefined;
    this.accompanimentType = undefined;
    this.isPrevHidden = false;
    this.isNextHidden = false;
    this.isLastStep = false;
    this.failedOrientation = false;
    this.nbSteps = OrientationFormViewComponent.MAX_STEP;
    this.isPageValid = false;
    this.filters = [];
  }

  /**
   * Indicators
   */
  private targetStructures(structureChoice: AbstractControl): OrientationIndicatorStructure[] {
    const targetStructures: OrientationIndicatorStructure[] = [];
    if (structureChoice?.value) {
      structureChoice.value.forEach((structure: Structure) => {
        targetStructures.push({
          id: structure._id,
          nom: structure.structureName,
          adresse: structure.address,
          zoneCTM: structure.categories.ctm?.join(',') ?? null,
          distance: structure.distance ?? null,
        });
      });
    }
    return targetStructures;
  }
  private displayTextNeedType(needType: NeedsTypes): string {
    switch (needType) {
      case NeedsTypes.equipmentAccess:
        return 'accès matériel';
      case NeedsTypes.equipmentBuy:
        return 'achat matériel';
      case NeedsTypes.onlineDemarch:
        return 'démarche en ligne';
      case NeedsTypes.baseSkills:
        return 'compétences';
      default:
        return '';
    }
  }
  private orientationType(filters: Filter[]): OrientationIndicatorType {
    const orientationType: OrientationIndicatorType = {
      typeOrientation: this.displayTextNeedType(this.needType),
      filtres: [],
      typeAccompagnement: this.accompanimentType,
    };
    filters.forEach((filter) => {
      orientationType.filtres.push(filter.text);
    });
    return orientationType;
  }
  private getProgress(): number {
    if (this.isLastStep) {
      return 100;
    }
    return Math.round((this.currentStep / this.nbSteps) * 100);
  }
  private async sendOrientationIndicator(orientationForm: FormGroup): Promise<void> {
    let isConnected = false;
    let profile: User = null;
    if (this.authService.isLoggedIn()) {
      isConnected = true;
      profile = await this.profilService.getProfile();
    }

    const orientationCompleted: boolean = this.isLastStep;
    const orientationIndicator: IOrientationIndicator = {
      origin: {
        id: undefined,
        prescripteur: this.utils.nullifyEmpty(profile?.job?.name),
        adresse: this.utils.nullifyEmpty(orientationForm?.get('address')?.value),
      },
      target: this.utils.nullifyEmpty(this.targetStructures(orientationForm.get('structureChoice'))),
      type: this.orientationType(this.filters),
      connected: isConnected,
      completed: orientationCompleted,
      progress: this.getProgress(),
    };

    this.indicatorService.createOrientationIndicator(orientationIndicator).subscribe();
  }

  /**
   * Confirmation modal
   */
  public canExit(): Promise<boolean> {
    // Avoid confirmation when user submit form and leave.
    if (this.canDeactivate || this.currentStep === null) {
      return Promise.resolve(true);
    } else {
      return new Promise((resolve) => this.showModal(resolve));
    }
  }
  private showModal(resolve: CanExitResolver): void {
    this.showConfirmationModal = true;
    this.resolve = resolve;
  }
  public hasRedirectionAccepted(hasAccept: boolean): void {
    this.resolve(hasAccept);
    if (hasAccept) {
      this.sendOrientationIndicator(this.structureOrientationForm ?? this.onlineDemarcheForm);
    }
    this.showConfirmationModal = false;
  }

  /**
   * Login modal
   */
  public goLogin(): void {
    this.router.navigate(['/connexion'], { queryParams: { returnUrl: '/orientation' } });
  }
  public handleClose(hasAccept: boolean): void {
    if (hasAccept) {
      this.goLogin();
    } else {
      this.showLoginModal = false;
    }
  }

  /**
   * Shortcuts
   */
  private checkSkipInfoScreen(goNext: boolean): void {
    // Info screen should be skipped for appointment, if coming from baseSkills
    if (this.tunnelingStepIs(AccompanimentTypes.appointment, AppointmentSteps.infoScreen)) {
      if (this.needType === NeedsTypes.baseSkills) {
        goNext ? this.goToNextStep() : this.goToPreviousStep();
      }
    }
  }
  private checkSkipCartoScreens(dir: 'prev' | 'next'): void {
    // When coming from carto or registry, we need to skip the carto steps
    if (this.isComingFromCartoOrRegistry()) {
      if (dir === 'prev' && this.tunnelingStepIs(AccompanimentTypes.appointment, AppointmentSteps.carto)) {
        this.currentStep = AppointmentSteps.infoScreen;
      } else if (dir === 'next' && this.tunnelingStepIs(AccompanimentTypes.appointment, AppointmentSteps.pmrAccess)) {
        this.currentStep = AppointmentSteps.makeAppointment;
      }
    }
  }
  private checkSkipStructureOrientator(goNext: boolean): void {
    // When only one structure, no need to set orientator
    if (
      this.tunnelingStepIs(AccompanimentTypes.structuresList, StructuresListSteps.structureOrientator) ||
      this.tunnelingStepIs(AccompanimentTypes.appointment, AppointmentSteps.structureOrientator) ||
      this.tunnelingStepIs(AccompanimentTypes.onlineMediation, OnlineMediationSteps.structureOrientator)
    ) {
      if (this.profile?.structuresLink?.length === 1) {
        goNext ? this.goToNextStep() : this.goToPreviousStep();
      }
    }
  }

  /**
   * Update states
   */
  private updateHiddenNavButtons(): void {
    // Only needs to hide in appointment / online mediation
    if (!this.isOnlineDemarchOrBaseSkills()) return;

    const isCartoStep = this.tunnelingStepIs(AccompanimentTypes.appointment, AppointmentSteps.carto);
    const isAppointmentSuccess = this.tunnelingStepIs(AccompanimentTypes.appointment, AppointmentSteps.rdvEnd);
    const isOnlineMediationSuccess = this.tunnelingStepIs(
      AccompanimentTypes.onlineMediation,
      OnlineMediationSteps.rdvEnd,
    );
    this.isPrevHidden = isAppointmentSuccess || isOnlineMediationSuccess;
    this.isNextHidden = isCartoStep;
  }
  private updateFullScreen(): void {
    this.fullScreen =
      this.tunnelingStepIs(AccompanimentTypes.structuresList, StructuresListSteps.structureChoice) ||
      this.tunnelingStepIs(AccompanimentTypes.appointment, AppointmentSteps.carto);
  }
  public updateLastStep(): void {
    const isStructureListOrientationRecap = this.tunnelingStepIs(
      AccompanimentTypes.structuresList,
      StructuresListSteps.orientationRecap,
    );
    const isAppointmentOrientationRecap = this.tunnelingStepIs(
      AccompanimentTypes.appointment,
      AppointmentSteps.orientationPrint,
    );
    const isOnlineMediationOrientationRecap = this.tunnelingStepIs(
      AccompanimentTypes.onlineMediation,
      OnlineMediationSteps.orientationPrint,
    );
    this.isLastStep =
      isStructureListOrientationRecap ||
      (this.isOnlineDemarchOrBaseSkills() && (isAppointmentOrientationRecap || isOnlineMediationOrientationRecap));
  }

  /**
   * Helpers
   */
  private commonStepIs(step: CommonSteps): boolean {
    return this.genericStep === GenericOrientationSteps.common && this.currentStep === step;
  }
  private tunnelingStepIs(
    accompanimentType: AccompanimentTypes,
    step: StructuresListSteps | AppointmentSteps | OnlineMediationSteps,
  ): boolean {
    return (
      this.genericStep === GenericOrientationSteps.accompanimentTunnel &&
      this.accompanimentType === accompanimentType &&
      this.currentStep === step
    );
  }
  public isOnlineDemarchOrBaseSkills(): boolean {
    return this.needType === NeedsTypes.onlineDemarch || this.needType === NeedsTypes.baseSkills;
  }
  private isComingFromCartoOrRegistry(): boolean {
    return Boolean(this.orientationService.rdvStructure || this.orientationService.rdvUser);
  }
  private shouldHandleAppointment(): boolean {
    return (
      this.tunnelingStepIs(AccompanimentTypes.appointment, AppointmentSteps.rdvEnd) ||
      this.tunnelingStepIs(AccompanimentTypes.onlineMediation, OnlineMediationSteps.rdvEnd)
    );
  }
  private setStepNumber(enumLength: number): void {
    this.nbSteps = enumLength / 2;
  }
}
