import { animate, animateChild, query, style, transition, trigger } from '@angular/animations';
import { Component, ElementRef, ViewChild } from '@angular/core';
import { NavigationEnd, Router } from '@angular/router';
import { Structure } from '../models/structure.model';
import { ProfileService } from '../profile/services/profile.service';
import { AuthService } from '../services/auth.service';

@Component({
  selector: 'app-header',
  templateUrl: './header.component.html',
  styleUrls: ['./header.component.scss'],
  animations: [
    trigger('slideInOut', [
      transition(':enter', [
        style({ left: '-350px', backgroundColor: 'rgb(51, 51, 51, 0)' }),
        animate('250ms ease-in', style({ left: '0', backgroundColor: 'rgb(51, 51, 51, 0.25)' })),
      ]),
      transition(':leave', [
        animate('250ms ease-in', style({ left: '-350px', backgroundColor: 'rgb(51, 51, 51, 0)' })),
      ]),
    ]),
    trigger('toggle', [
      transition(':enter', [query('@fadeInOut', animateChild())]),
      transition(':leave', [query('@fadeInOut', animateChild())]),
    ]),
    trigger('fadeInOut', [
      transition(':enter', [style({ opacity: '0' }), animate('200ms ease-in', style({ opacity: '1' }))]),
      transition(':leave', [animate('200ms ease-in', style({ opacity: '0' }))]),
    ]),
  ],
})
export class HeaderComponent {
  public showMenu = false;
  public showProfileMenu = false;
  public currentRoute = '';
  public formRoute = '/create-structure';
  public returnUrl = null;
  public dataConsentPendingStructures: Structure[];
  private displayDataShare = false;
  private loadingDataShare = false;

  @ViewChild('clickableLogo') clickableLogoDiv: ElementRef;

  constructor(
    private authService: AuthService,
    private profileService: ProfileService,
    private router: Router,
  ) {
    this.router.events.subscribe((event) => {
      if (event instanceof NavigationEnd) {
        this.currentRoute = event.url;
      }
    });
  }

  public isActive(route: string): 'page' | '' {
    return this.router.url === route ? 'page' : '';
  }
  public goToHome(): void {
    this.closeMenu();
    this.router.navigateByUrl('/home');
  }
  public goToProfile(): void {
    this.showProfileMenu = false;
    this.router.navigateByUrl('/profil');
  }

  public toggleMenu(): void {
    this.showMenu = !this.showMenu;
  }
  public closeMenu(): void {
    this.showMenu = false;
  }

  public openProfileMenu(): void {
    this.showProfileMenu = true;
  }
  public closeProfileMenu(): void {
    this.showProfileMenu = false;
  }
  public clickProfileButton(): void {
    this.closeMenu();
    if (this.isLoggedIn) {
      this.openProfileMenu();
    } else {
      this.goToLoginPage();
    }
  }

  public focusLogo(): void {
    if (this.clickableLogoDiv) {
      this.clickableLogoDiv.nativeElement.focus();
    }
  }

  public get isLoggedIn(): boolean {
    return this.authService.isLoggedIn();
  }

  public get isDisplayDataShare(): boolean {
    if (this.displayDataShare) return true;

    if (this.isLoggedIn && !this.loadingDataShare) {
      this.loadingDataShare = true;
      this.profileService.getAllDataConsentPendingStructures().subscribe((dataConsentPendingStructures) => {
        if (dataConsentPendingStructures.length) {
          this.displayDataShare = true;
          this.dataConsentPendingStructures = dataConsentPendingStructures;
          return this.displayDataShare;
        }
      });
    }
    return false;
  }
  public goToLoginPage(): void {
    this.router.navigateByUrl('connexion');
  }
  public get isAdmin(): boolean {
    return this.profileService.isAdmin();
  }

  public get displayName(): string {
    return this.authService.getUserNameDisplay();
  }

  public get displayFullName(): string {
    return this.authService.getUserFullNameDisplay();
  }

  public displayLogo(): boolean {
    return this.formRoute !== this.currentRoute;
  }

  public logout(): void {
    this.authService.logout();
  }
}
