import { Component, OnInit } from '@angular/core';
import { AbstractControl, UntypedFormBuilder, UntypedFormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router, UrlSegment } from '@angular/router';
import { combineLatest, lastValueFrom } from 'rxjs';
import { map } from 'rxjs/operators';
import { Structure } from '../models/structure.model';
import { AuthService } from '../services/auth.service';
import { NotificationService } from '../services/notification.service';
import { CustomRegExp } from '../utils/CustomRegExp';

@Component({
  selector: 'app-login',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.scss'],
})
export class LoginComponent implements OnInit {
  public loginForm: UntypedFormGroup;
  public loading = false;
  public submitted = false;
  public authFailed = false;
  public isUnverifiedEmail = false;
  public isWelcome = false;
  public userId: string;
  public token: string;
  public returnUrl: string;
  public structure: Structure;
  public verificationSuccess = false;
  public verificationIssue = false;

  constructor(
    private formBuilder: UntypedFormBuilder,
    private router: Router,
    private authService: AuthService,
    private activatedRoute: ActivatedRoute,
    private notificationService: NotificationService,
  ) {}

  ngOnInit(): void {
    // Subscribe to routing for user
    const urlParametrs = combineLatest([this.activatedRoute.parent.url, this.activatedRoute.queryParams]).pipe(
      map((results) => ({
        url: results[0],
        token: results[1].token,
      })),
    );

    urlParametrs.subscribe((routeParams: { url: UrlSegment[]; token: string }) => {
      // Write code to use routeParams and queryParams.
      if (routeParams.url[0].path === 'users' && routeParams.url[1].path === 'verify' && routeParams.url[2]) {
        this.isWelcome = true;
        this.userId = routeParams.url[2].path;
        this.token = routeParams.token;
        this.sendVerification();
      }
    });

    this.activatedRoute.queryParams.subscribe((params) => {
      this.returnUrl = params.returnUrl;
    });

    // login form
    this.loginForm = this.formBuilder.group({
      email: ['', [Validators.required, Validators.pattern(CustomRegExp.EMAIL)]],
      password: ['', [Validators.required, Validators.pattern(CustomRegExp.PASSWORD)]],
    });
  }

  // getter for form fields
  get f(): Record<string, AbstractControl> {
    return this.loginForm.controls;
  }

  public switchToResetPassword(): void {
    this.router.navigate(['/mot-de-passe-oublie']);
  }

  public async onSubmit(): Promise<void> {
    this.submitted = true;

    // stop here if form is invalid
    if (this.loginForm.invalid) {
      return;
    }
    this.loading = true;
    this.authFailed = false;
    this.isUnverifiedEmail = false;
    try {
      await lastValueFrom(this.authService.login(this.f.email.value, this.f.password.value));

      if (this.isWelcome) {
        this.router.navigateByUrl('formulaire/profil');
      } else {
        if (this.returnUrl) {
          this.router.navigateByUrl(this.returnUrl);
        } else {
          this.router.navigateByUrl('actualites');
        }
      }
    } catch (e) {
      this.loading = false;
      if (e.status === 418) {
        this.isUnverifiedEmail = true;
      } else {
        this.authFailed = true;
      }
    }
  }

  public goToAccountCreation(): void {
    this.router.navigateByUrl('formulaire/compte');
  }

  public getLoginStatusText(): string {
    if (this.f.email.value && this.f.email.invalid) {
      return 'L’identifiant doit être un email valide';
    }
    if (this.isUnverifiedEmail) {
      return 'Votre email n’a jamais été validé';
    }
    return null;
  }

  public getPasswordStatusText(): string {
    if (this.f.password.invalid) {
      return 'Le mot de passe doit obligatoirement contenir : 8&nbsp;caractères, une majuscule, une minuscule, un caractère spécial et un chiffre';
    }
    if (this.authFailed) {
      return 'Identifiant ou mot de passe invalide';
    }
    return null;
  }

  public onChange(): void {
    this.authFailed = false;
    this.isUnverifiedEmail = false;
  }

  public resendConfirmationEmail(): void {
    this.isUnverifiedEmail = false;
    this.loading = true;
    this.authService.resendEmail(this.f.email.value, this.f.password.value).subscribe({
      next: () => {
        this.loading = false;
        this.notificationService.showSuccess(`L'email de confirmation a été renvoyé à ${this.f.email.value}`);
      },
      error: (error) => {
        this.loading = false;
        if (error.status === 401) {
          this.authFailed = true;
          this.notificationService.showError(
            `Identifiant ou mot de passe invalide`,
            `L'email de confirmation n'a pas pu être renvoyé`,
          );
        } else {
          console.log(`L'email de confirmation n'a pas pu être renvoyé: ${error.message}`);
          this.notificationService.showErrorPleaseRetry(`L'email de confirmation n'a pas pu être renvoyé`);
        }
      },
    });
  }

  private sendVerification(): void {
    this.authService.verifyUser(this.userId, this.token).subscribe({
      error: () => {
        this.verificationIssue = true;
      },
    });
  }
}
