import { HttpErrorResponse } from '@angular/common/http';
import { Component, Input, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { lastValueFrom } from 'rxjs';
import { Employer } from '../../models/employer.model';
import { Job } from '../../models/job.model';
import { User } from '../../models/user.model';
import { AuthService } from '../../services/auth.service';
import { NotificationService } from '../../services/notification.service';
import { CustomRegExp } from '../../utils/CustomRegExp';
import { CanExitResolver, Utils } from '../../utils/utils';
import { ProfileService } from '../services/profile.service';

enum tabsEnum {
  details,
  credentials,
  employer,
  description,
}

enum showPasswordEnum {
  oldPassword = 'oldPassword',
  newPassword = 'newPassword',
  newPasswordConfirm = 'newPasswordConfirm',
}

type passwordStatuses = 'error' | 'info' | 'success';

@Component({
  selector: 'app-edit',
  templateUrl: './edit.component.html',
  styleUrls: ['./edit.component.scss'],
})
export class EditComponent implements OnInit {
  public tabsEnum = tabsEnum;
  public currentTab = tabsEnum.details;

  @Input() userProfile: User;
  public initialUserProfile: User;
  public newEmail = '';
  public newEmailConfirm = '';
  public oldPassword = '';
  public newPassword = '';
  public newPasswordConfirm = '';
  public isShowPassword = {
    oldPassword: false,
    newPassword: false,
    newPasswordConfirm: false,
  };
  public ShowPassword = showPasswordEnum;
  public selectedJob: Job;
  public jobName: string;
  public isNewJob: boolean;
  public selectedEmployer: Employer;
  public employerName: string;
  public isNewEmployer: boolean;
  public selectedRdvChoice: boolean;
  public isAlreadySearching = false;
  public hasPersonalOffer = false;
  public errorChangeEmail = false;
  public statusCode = 200;

  // Modal canExit var
  public emailModal = false;
  public passwordModal = false;
  public deleteAccountModal = false;
  public pendingChangesModal = false;
  private resolve: CanExitResolver;
  private canDeactivate = false;
  public appointmentModal = false;

  constructor(
    public profileService: ProfileService,
    private notificationService: NotificationService,
    private authService: AuthService,
    private utils: Utils,
    private router: Router,
  ) {}

  ngOnInit(): void {
    if (history.state.data === 'description') {
      this.currentTab = tabsEnum.description;
    }
    this.profileService.getProfile().then((profile) => {
      if (profile.hasOwnProperty('withAppointment')) this.selectedRdvChoice = profile.withAppointment;
      this.userProfile = new User(profile);
      this.userProfile.description = this.userProfile.description || '';
      this.initialUserProfile = new User({ ...profile });
      this.selectedEmployer = { ...profile.employer };
      this.employerName = this.selectedEmployer?.name;
      this.selectedJob = { ...profile.job };
      this.jobName = this.selectedJob?.name;
      this.hasPersonalOffer = profile.job?.hasPersonalOffer;
    });
  }

  public goBack(): void {
    history.back();
  }

  public getTabsNames(): string[] {
    return ['Coordonnées', 'Email et mot de passe', 'Employeur et fonction', 'Description'];
  }

  public getTabsIds(): string[] {
    return ['detailsTab', 'credentials', 'employerJob', 'descriptionTab'];
  }

  public changeTab(tab: number): void {
    this.currentTab = tab;
  }

  public phoneValid(): boolean {
    return Boolean(this.userProfile.phone.match(CustomRegExp.PHONE));
  }

  public nameValid(): boolean {
    return this.userProfile.name !== '';
  }

  public surnameValid(): boolean {
    return this.userProfile.surname !== '';
  }

  public descriptionValid(): boolean {
    return this.userProfile.description?.length <= 500;
  }

  public emailValid(email: string): boolean {
    return Boolean(RegExp(CustomRegExp.EMAIL).exec(email));
  }

  public getNewEmailStatus(): boolean {
    return this.emailValid(this.newEmail);
  }

  public getNewEmailStatusText(): string {
    return `Nouvel email ${this.getNewEmailStatus() ? 'valide' : 'invalide'}`;
  }

  public getNewEmailConfirmStatus(): boolean {
    return this.emailValid(this.newEmail) && this.newEmailConfirm === this.newEmail;
  }

  public getNewEmailConfirmStatusText(): string {
    if (!this.newEmail || !this.newEmailConfirm) return '';
    return `Confirmation du nouvel email ${this.getNewEmailConfirmStatus() ? 'valide' : 'invalide'}`;
  }

  public passwordValid(password: string): boolean {
    return !!(
      password.match(CustomRegExp.SPECHAR) &&
      password.match(CustomRegExp.DIGIT) &&
      password.match(CustomRegExp.UPPERCASE) &&
      password.match(CustomRegExp.LOWERCASE) &&
      password.match(CustomRegExp.MINLENGTH)
    );
  }

  public passwordValidConfirm(passwordConfirm: string, password: string): boolean {
    return this.passwordValid(passwordConfirm) && passwordConfirm === password;
  }

  public getPasswordStatus(password: string, compareTo?: string): passwordStatuses {
    if (this.utils.isNullOrEmpty(password)) return 'info';

    const isValid = compareTo ? this.passwordValidConfirm(compareTo, password) : this.passwordValid(password);
    return isValid ? 'success' : 'error';
  }

  public getOldPasswordStatusText(status: passwordStatuses): string {
    const statusTexts = {
      error: 'Mot de passe invalide.',
      success: 'Champ rempli.',
    };

    return statusTexts[status];
  }

  public getPasswordStatusText(status: passwordStatuses): string {
    const statusTexts = {
      info: 'Le mot de passe doit obligatoirement contenir : 8 caractères, une majuscule, un caractère spécial.',
      error: 'Le mot de passe doit obligatoirement contenir : 8 caractères, une majuscule, un caractère spécial.',
      success: 'Mot de passe valide.',
    };

    return statusTexts[status];
  }

  public getNewPasswordConfirmStatusText(status: passwordStatuses): string {
    const statusTexts = {
      error: 'Confirmation du nouveau mot de passe invalide.',
      success: 'Confirmation du nouveau mot de passe valide.',
    };

    return statusTexts[status];
  }

  getStatus(success: boolean): 'error' | 'success' {
    return success ? 'success' : 'error';
  }

  public cancel(): void {
    this.userProfile = { ...this.initialUserProfile };
  }

  public showEmailModal(): void {
    this.emailModal = true;
  }

  public showPasswordModal(): void {
    this.passwordModal = true;
  }

  public showDeleteAccountModal(): void {
    this.deleteAccountModal = true;
  }

  public showPassword(key: showPasswordEnum): void {
    this.isShowPassword[key] = !this.isShowPassword[key];
  }

  public closeModal(): void {
    this.oldPassword = '';
    this.newPassword = '';
    this.newPasswordConfirm = '';
    this.emailModal = false;
    this.passwordModal = false;
    this.deleteAccountModal = false;
  }

  public isPageValid(): boolean {
    if (this.currentTab === tabsEnum.details) {
      return this.coordsHaveChanged() && this.phoneValid() && this.nameValid() && this.surnameValid();
    } else if (this.currentTab === tabsEnum.credentials) {
      if (this.emailModal) {
        return this.emailValid(this.newEmail) && this.newEmail === this.newEmailConfirm;
      } else if (this.passwordModal) {
        return this.passwordValid(this.newPassword) && this.newPassword === this.newPasswordConfirm;
      }
    } else if (this.currentTab === tabsEnum.employer) {
      return this.employerAndJobValid() && this.employerOrJobHasChanged();
    } else if (this.descriptionHasChanged()) {
      return this.descriptionValid();
    }
  }

  public confirm(): void {
    this.canDeactivate = true;
    if (this.currentTab === tabsEnum.details) {
      this.confirmDetails();
    } else if (this.currentTab === tabsEnum.credentials) {
      if (this.emailModal) {
        this.confirmNewEmail();
      } else if (this.passwordModal) {
        this.confirmNewPassword();
      }
    } else if (this.currentTab === tabsEnum.employer) {
      this.confirmEmployer();
    } else if (this.currentTab === tabsEnum.description) {
      this.confirmDescription();
    }
  }

  public confirmDetails(): void {
    this.profileService
      .updateDetails({
        name: this.userProfile.name,
        surname: this.userProfile.surname,
        phone: this.userProfile.phone,
      })
      .subscribe((user: User) => {
        this.notificationService.showSuccess('Vos informations ont bien été enregistrées');
        // Update localStorage
        const updatedUser = {
          ...this.authService.userSubject.value,
          name: user.name,
          surname: user.surname,
          phone: user.phone,
        };
        localStorage.setItem('user', JSON.stringify(updatedUser));
        this.authService.userSubject.next(updatedUser);
        this.updateInitialProfile();
        this.router.navigate(['/profil']);
      });
  }

  public async confirmEmployer(): Promise<void> {
    if (this.isNewJob) {
      await lastValueFrom(this.profileService.createJob(this.selectedJob));
    }
    if (this.isNewEmployer) {
      await lastValueFrom(this.profileService.createEmployer(this.selectedEmployer));
    }

    this.profileService
      .updateProfile(this.selectedEmployer.name, this.selectedJob.name, this.selectedRdvChoice)
      .subscribe({
        next: () => {
          this.router.navigate(['/profil']);
          this.notificationService.showSuccess('Vos informations ont bien été enregistrées');
        },
        error: () => {
          this.notificationService.showError("Vos informations n'ont pas été enregistrées", 'Une erreur est survenue');
          this.canDeactivate = false;
        },
      });
  }

  public onRdvRadioChange(value: boolean): void {
    this.selectedRdvChoice = value;
  }

  public confirmDescription(): void {
    this.profileService.updateDescription(this.userProfile.description).subscribe(() => {
      this.notificationService.showSuccess('Vos informations ont bien été enregistrées');
      this.updateInitialProfile();
      this.router.navigate(['/profil']);
    });
  }

  // Updates initialProfile so the button "Valider" is disabled after the user sends new values
  private updateInitialProfile(): void {
    this.initialUserProfile = { ...this.userProfile };
  }

  public confirmNewEmail(): void {
    if (this.emailValid(this.newEmail) && this.newEmail === this.newEmailConfirm) {
      this.profileService.changeEmail(this.newEmail, this.userProfile.email).subscribe({
        next: () => {
          this.closeModal();
          this.notificationService.showSuccess('Veuillez confirmer votre nouvelle adresse grâce au mail envoyé');
          this.newEmail = '';
          this.newEmailConfirm = '';
        },
        error: (error: HttpErrorResponse) => {
          this.statusCode = error.status;
          this.closeModal();
          this.errorChangeEmail = true;
        },
        complete: () => {
          this.canDeactivate = false;
        },
      });
    }
  }

  public confirmNewPassword(): void {
    if (this.passwordValid(this.newPassword) && this.newPassword === this.newPasswordConfirm) {
      this.profileService.changePassword(this.newPassword, this.oldPassword).subscribe({
        next: () => {
          this.notificationService.showSuccess('Votre mot de passe a bien été modifié');
          this.closeModal();
        },
        error: (error: HttpErrorResponse) => {
          if (error.status === 401) {
            this.notificationService.showError('Ancien mot de passe invalide');
          } else {
            this.notificationService.showErrorPleaseRetry('Une erreur est survenue');
          }
        },
        complete: () => {
          this.oldPassword = '';
          this.isShowPassword.oldPassword = false;
          this.canDeactivate = false;
        },
      });
    }
  }

  public async confirmDeleteAccount(): Promise<void> {
    try {
      await lastValueFrom(this.authService.login(this.userProfile.email, this.oldPassword));

      await lastValueFrom(this.profileService.deleteProfile());
      this.closeModal();

      this.notificationService.showSuccess('Votre compte a bien été supprimé');
      setTimeout(() => this.authService.logout(), 3000); // Let time for user to view notification before logout
    } catch (e) {
      if (e.status === 401) {
        this.notificationService.showError('Mot de passe invalide');
      } else {
        this.notificationService.showErrorPleaseRetry('Une erreur est survenue');
      }
    }
  }

  // Jobs
  public selectJob(job: Job): void {
    this.selectedJob = job;
    this.jobName = this.selectedJob?.name;
    this.isNewJob = false;
    this.hasPersonalOffer = job?.hasPersonalOffer;
  }

  public onCreateValueChangeJob(value: string): void {
    this.selectedJob = new Job({ name: value, validated: false, hasPersonalOffer: true });
    this.isNewJob = true;
    this.hasPersonalOffer = this.selectedJob?.hasPersonalOffer;
  }

  // Employer
  public selectEmployer(employer: Employer): void {
    this.selectedEmployer = employer;
    this.employerName = this.selectedEmployer?.name;
    this.isNewEmployer = false;
  }

  public onCreateValueChangeEmployer(value: string): void {
    this.selectedEmployer = new Employer({ name: value, validated: false });
    this.isNewEmployer = true;
  }

  public canExit(): Promise<boolean> {
    if (this.hasPendingChanges() && !this.canDeactivate) {
      return new Promise((resolve) => this.showPendingChangesModal(resolve));
    }
    if (this.hasPersonalOffer && this.selectedRdvChoice === undefined) {
      return new Promise(() => this.showAppointmentModal());
    }
    return Promise.resolve(true);
  }

  private coordsHaveChanged(): boolean {
    return (
      this.userProfile.name !== this.initialUserProfile.name ||
      this.userProfile.surname !== this.initialUserProfile.surname ||
      this.userProfile.phone !== this.initialUserProfile.phone
    );
  }

  private employerAndJobValid(): boolean {
    // Check if mandatory fields are set (with rdv defined if needed)
    if (!this.selectedEmployer || !this.selectedJob) {
      return false;
    }
    if (this.hasPersonalOffer && this.selectedRdvChoice === undefined) {
      return false;
    }
    return true;
  }

  private employerOrJobHasChanged(): boolean {
    // Check if any field has changed : employer or job, or rdv choice if applicable
    return (
      this.selectedEmployer?.name !== this.initialUserProfile.employer?.name ||
      this.selectedJob?.name !== this.initialUserProfile.job?.name ||
      (this.hasPersonalOffer && this.selectedRdvChoice !== this.initialUserProfile.withAppointment)
    );
  }

  private descriptionHasChanged(): boolean {
    return this.userProfile.description !== (this.initialUserProfile.description ?? '');
  }

  public hasPendingChanges(): boolean {
    return this.coordsHaveChanged() || this.employerOrJobHasChanged() || this.descriptionHasChanged();
  }

  public showPendingChangesModal(resolve: CanExitResolver): void {
    this.pendingChangesModal = true;
    this.resolve = resolve;
  }
  public hasPendingChangesRedirectionAccepted(hasAccept: boolean): void {
    this.resolve(hasAccept);
    this.pendingChangesModal = false;
  }

  private showAppointmentModal(): void {
    this.appointmentModal = true;
  }
  public closeAppointmentModal(): void {
    this.appointmentModal = false;
  }
}
