import { Component, OnInit } from '@angular/core';
import { UntypedFormControl, UntypedFormGroup } from '@angular/forms';
import { ActivatedRoute, Data, Router } from '@angular/router';
import { CategoriesToggle } from '../../models/categoriesToggle.model';
import { NotificationService } from '../../services/notification.service';
import { CategoryEnum } from '../../shared/enum/category.enum';
import { Category } from '../../structure-list/models/category.model';
import { SearchService } from '../../structure-list/services/search.service';
import { PersonalOffer } from './../../models/personalOffer.model';
import { PersonalOfferService } from './../../services/personal-offer.service';

enum tabsEnum {
  onlineProcedures,
  digitalSkills,
}

@Component({
  selector: 'app-personal-offer-edition',
  templateUrl: './personal-offer-edition.component.html',
  styleUrls: ['./personal-offer-edition.component.scss'],
})
export class PersonalOfferEditionComponent implements OnInit {
  public tabsEnum = tabsEnum;
  public currentTab: tabsEnum = tabsEnum.onlineProcedures;

  public personalOffer: PersonalOffer;
  public personalOfferForm: UntypedFormGroup = null;
  private initialPersonalOffer;
  public onlineProcedures: Category;
  public trainingCategories: CategoriesToggle[] = [];
  public deleteOfferModal: boolean;
  public structureName: string;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private searchService: SearchService,
    private personalOfferService: PersonalOfferService,
    private notificationService: NotificationService,
  ) {}

  ngOnInit(): void {
    this.route.data.subscribe(async (data: Data & { personalOffer: PersonalOffer }) => {
      if (data.personalOffer) {
        await this.setCategories();
        this.personalOffer = data.personalOffer;
        this.personalOfferForm = this.createPersonalOfferForm(this.personalOffer);
        this.initialPersonalOffer = this.personalOfferForm.value;
      }
    });
    this.structureName = history.state.structureName;
  }

  public getTabsNames(): string[] {
    return ['Démarches en ligne', 'Compétences numériques'];
  }

  public getTabsIds(): string[] {
    return ['onlineProcedures', 'digitalSkills'];
  }

  public changeTab(tab: number): void {
    this.currentTab = tab;
  }

  private createPersonalOfferForm(personalOfferState): UntypedFormGroup {
    return new UntypedFormGroup({
      categories: new UntypedFormGroup({
        onlineProcedures: new UntypedFormControl(personalOfferState.categories.onlineProcedures),
        baseSkills: new UntypedFormControl(personalOfferState.categories.baseSkills),
        advancedSkills: new UntypedFormControl(personalOfferState.categories.advancedSkills),
      }),
    });
  }

  async setCategories(): Promise<void> {
    const categories = await this.searchService.getCategories().toPromise();
    categories.forEach((categ) => {
      switch (categ.id) {
        case CategoryEnum.onlineProcedures: {
          // We sort modules alphabetically, except for the 'autres' module which is always at the end
          this.onlineProcedures = categ;
          const othersModule = this.onlineProcedures.modules.find((module) => module.id === 'autres');
          const sortedModules = this.onlineProcedures.modules
            .filter((module) => module.id !== 'autres')
            .sort((a, b) => a.name.localeCompare(b.name));
          if (othersModule) sortedModules.push(othersModule);
          this.onlineProcedures.modules = sortedModules;
          break;
        }
        case CategoryEnum.baseSkills:
        case CategoryEnum.advancedSkills: {
          this.trainingCategories.push({ category: categ, openned: false });
          break;
        }
        default:
          break;
      }
    });
  }

  public cancel(): void {
    this.personalOfferForm = this.createPersonalOfferForm(this.initialPersonalOffer);
  }

  public confirm(): void {
    // If all options are unchecked, delete offer instead of saving it
    const categories: Record<string, string[]> = this.personalOfferForm.get('categories').value;
    const isFormEmpty = Object.keys(categories).every((category) => !categories[category].length);
    if (isFormEmpty) {
      this.showDeleteOfferModal();
    } else {
      this.personalOfferService
        .updatePersonalOffer(this.personalOffer._id, this.personalOfferForm.get('categories').value)
        .subscribe(() => {
          this.notificationService.showSuccess('Vos informations ont bien été enregistrées');
          this.initialPersonalOffer = this.personalOfferForm.value;
          this.personalOfferForm.markAsPristine();
        });
      this.router.navigateByUrl('/profil');
    }
  }

  public setTrainingsFromCategories(categories: Category[]): void {
    for (const categorie of categories) {
      const moduleIds: string[] = categorie.modules.map((module) => module.id);
      if (this.personalOfferForm.get('categories').get(categorie.id)) {
        this.personalOfferForm.get('categories').get(categorie.id).patchValue(moduleIds);
        this.personalOfferForm.get('categories').get(categorie.id).markAsDirty();
      }
    }
  }

  public showDeleteOfferModal(): void {
    this.deleteOfferModal = true;
  }
  public closeModal(): void {
    this.deleteOfferModal = false;
  }

  public confirmDeleteOffer(): void {
    this.personalOfferService.deletePersonalOffer(this.personalOffer._id).subscribe(
      () => {
        this.notificationService.showSuccess('Votre offre d’accompagnements a bien été supprimée.');
      },
      (err) => {
        this.notificationService.showError(`${err.error.message}`, 'Une erreur est survenue');
      },
    );
    this.router.navigate(['/profil']);
  }

  public goBack(): void {
    history.back();
  }
}
