import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { Router } from '@angular/router';
import { DateTime } from 'luxon';
import { Structure } from '../../models/structure.model';
import { StructureWithOwners } from '../../models/structureWithOwners.model';
import { CollapseType } from '../../shared/components/collapse/collapse.type';
import { SearchService } from '../../structure-list/services/search.service';
import { FormUtils } from '../../utils/formUtils';
import { Utils } from '../../utils/utils';
import { PersonalOffer } from './../../models/personalOffer.model';
import { User } from './../../models/user.model';
import { UserService } from './../../services/user.service';

@Component({
  selector: 'app-profile-structure',
  templateUrl: './profile-structure.component.html',
  styleUrls: ['./profile-structure.component.scss'],
})
export class ProfileStructureComponent implements OnInit {
  @Input() public structureWithOwners: StructureWithOwners;
  @Input() public userProfile: User;
  @Input() public isPublic: boolean;
  @Input() public isPending = false;
  @Input() public joinRequestDate: string | null;
  @Output() cancelJoin = new EventEmitter<string>();

  public membersWithJobWithPO: User[] = [];
  public structureForm: FormGroup;
  public showDetails = false;
  public addMemberModalOpened = false;
  public structure: Structure;
  public personalOffer: PersonalOffer;

  constructor(
    private router: Router,
    private userService: UserService,
    private searchService: SearchService,
    public utils: Utils,
  ) {}

  async ngOnInit(): Promise<void> {
    this.structureForm = new FormUtils().createStructureForm(this.structureWithOwners.structure, true);
    this.structureWithOwners.owners.forEach((owner) => {
      this.userService.getUser(owner._id).subscribe((user) => {
        if (user.job?.hasPersonalOffer) this.membersWithJobWithPO.push(user);
      });
    });
    this.structure = new Structure(this.structureWithOwners.structure);
    this.personalOffer = await this.getSharedPersonalOffer();
  }

  public async getSharedPersonalOffer(): Promise<PersonalOffer> {
    // Check if user has personal offers

    if (
      !this.userProfile?.personalOffers ||
      !this.userProfile?.job?.hasPersonalOffer ||
      this.userProfile?.personalOffers?.length === 0
    ) {
      return null;
    }

    // Check if structure has personal offers
    if (this.structure.personalOffers.length === 0) return null;
    // Return personal offer if the user has one in this structure
    const personalOffer = this.structure.personalOffers.filter((structureOffer) =>
      this.userProfile.personalOffers.includes(structureOffer._id),
    )[0];
    // Get categories labels
    if (personalOffer) {
      const categories = await this.searchService.getCategories().toPromise();
      return this.utils.setServiceCategories(categories, personalOffer);
    }
  }

  public goBack(): void {
    history.back();
  }
  public goToHome(): void {
    this.router.navigateByUrl('/home');
  }
  public goToDetail(): void {
    this.router.navigate([], { queryParams: { structure: this.structure.permalink } });
  }
  public goToOffer(): void {
    this.router.navigate(['/formulaire/offre-personnelle'], { state: { structure: this.structure } });
  }
  public goToEditStructure(): void {
    this.router.navigateByUrl(`/profil/edition-structure/${this.structure.permalink}`);
  }
  public goToStructureMembersManagement(): void {
    this.router.navigateByUrl(`/profil/gestion-membres/${this.structure.permalink}`);
  }
  public isValid(): boolean {
    return this.structureForm.valid;
  }
  public getStructureTypeIcon(): string {
    return this.structure?.getTypeStructureIcon();
  }

  public getAddress(): string {
    const address = this.structure.address;
    return address.numero
      ? address.numero + ' ' + address.street + ' - ' + address.commune
      : address.street + ' - ' + address.commune;
  }

  public getFormattedDate(): string {
    if (this.joinRequestDate) {
      return DateTime.fromISO(this.joinRequestDate, { zone: 'Europe/Paris' }).toFormat('dd/MM/yyyy');
    }
  }

  public handleCancelJoin(idStructure: string): void {
    this.cancelJoin.emit(idStructure);
  }

  get collapseVariant(): CollapseType {
    return this.isPending ? CollapseType.Warning : CollapseType.Regular;
  }
}
