import { Location } from '@angular/common';
import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Observable, forkJoin } from 'rxjs';
import { catchError, map, tap } from 'rxjs/operators';
import { StructureWithOwners } from '../models/structureWithOwners.model';
import { User } from '../models/user.model';
import { NotificationService } from '../services/notification.service';
import { StructureService } from '../services/structure.service';
import { Utils } from '../utils/utils';
import { UserService } from './../services/user.service';
import { ProfileService } from './services/profile.service';

@Component({
  selector: 'app-profile',
  templateUrl: './profile.component.html',
  styleUrls: ['./profile.component.scss'],
})
export class ProfileComponent implements OnInit {
  public userProfile: User;
  public structures: StructureWithOwners[] = [];
  public pendingStructures: StructureWithOwners[] = [];
  public isPublic: boolean;

  constructor(
    private profileService: ProfileService,
    private structureService: StructureService,
    private userService: UserService,
    private route: ActivatedRoute,
    private router: Router,
    private location: Location,
    public utils: Utils,
    private notificationService: NotificationService,
  ) {}

  ngOnInit(): void {
    this.route.params.subscribe((urlParams) => {
      const userPermalink = urlParams.userPermalink || '';
      if (userPermalink) {
        this.isPublic = true;
        this.userService
          .getUserByPermalink(userPermalink)
          .pipe(
            map((res) => res),
            catchError(() => {
              this.router.navigate(['/home']);
              return new Observable<User>();
            }),
          )
          .subscribe((user) => {
            this.userProfile = new User(user);
            this.getStructuresFromProfile();
            this.getPendingStructuresFromProfile();
          });
      } else {
        this.isPublic = false;
        this.profileService.getProfile().then((profile) => {
          this.userProfile = new User(profile);
          this.getStructuresFromProfile();
          this.getPendingStructuresFromProfile();
        });
      }
    });
  }

  private getStructuresFromProfile(): void {
    const structures$: Observable<any>[] = [];
    this.structures = [];

    this.userProfile.structuresLink.forEach((structureId) => {
      structures$.push(
        this.structureService.getStructureWithOwners(structureId).pipe(
          tap((structure) => {
            this.structures.push(structure);
          }),
        ),
      );
    });
    forkJoin(structures$).subscribe(() => {
      this.structures.sort((a, b) => a.structure.structureName.localeCompare(b.structure.structureName));
    });
  }
  private getPendingStructuresFromProfile(): void {
    const structures$: Observable<any>[] = [];
    this.structures = [];
    this.userProfile.pendingStructuresLink.forEach((pending) => {
      structures$.push(
        this.structureService.getStructureWithOwners(pending.id).pipe(
          tap((structure) => {
            this.pendingStructures.push(structure);
          }),
        ),
      );
    });
    forkJoin(structures$).subscribe(() => {
      this.pendingStructures.sort((a, b) => a.structure.structureName.localeCompare(b.structure.structureName));
    });
  }

  public goBack(): void {
    this.location.back();
  }

  public goToOrientation(): void {
    this.router.navigate(['/orientation'], { state: { rdvUser: this.userProfile } });
  }
  public goToProfileEdit(tabToSelect?: string): void {
    if (tabToSelect) {
      this.router.navigateByUrl('/profil/edition', { state: { data: tabToSelect } });
    } else {
      this.router.navigateByUrl('/profil/edition');
    }
  }
  public goToStructuresManagement(): void {
    this.router.navigateByUrl('/profil/gestion-structures');
  }
  public goToFormStructure(): void {
    this.router.navigateByUrl('/formulaire/structure');
  }
  public cancelJoin(idStructure: string): void {
    this.userService.cancelJoin(idStructure, this.userProfile._id).subscribe({
      next: () => {
        const index = this.pendingStructures.map((s) => s.structure._id).indexOf(idStructure);
        this.pendingStructures.splice(index, 1);
        this.notificationService.showSuccess('La demande a été annulée avec succès');
      },
      error: (err) => {
        this.notificationService.showError(`${err.error.message}`, 'Une erreur est survenue');
        console.error(err);
      },
    });
  }
}
