import { animate, AUTO_STYLE, state, style, transition, trigger } from '@angular/animations';
import { Component, OnInit } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { DateTime } from 'luxon';
import { lastValueFrom } from 'rxjs';
import { structureFormStep } from '../../form/form-view/structure-form/structureFormStep.enum';
import { Owner } from '../../models/owner.model';
import { Structure } from '../../models/structure.model';
import { NotificationService } from '../../services/notification.service';
import { StructureService } from '../../services/structure.service';
import { CategoryEnum } from '../../shared/enum/category.enum';
import { WindowScrollService } from '../../shared/service/windowScroll.service';
import { AccessModality } from '../../structure-list/enum/access-modality.enum';
import { Equipment } from '../../structure-list/enum/equipment.enum';
import { Category } from '../../structure-list/models/category.model';
import { SearchService } from '../../structure-list/services/search.service';
import { FreeWorkshop } from '../../structure/enums/freeWorkshop.enum';
import { FormUtils, IStructureSummary } from '../../utils/formUtils';
import { Utils } from '../../utils/utils';
import { ProfileService } from '../services/profile.service';

@Component({
  selector: 'app-structure-edition-summary',
  templateUrl: './structure-edition-summary.component.html',
  styleUrls: ['./structure-edition-summary.component.scss'],
  animations: [
    trigger('show', [
      state('true', style({ height: AUTO_STYLE, visibility: AUTO_STYLE, margin: '8px 0' })),
      state('false', style({ height: '0px', visibility: 'hidden', margin: '0' })),
      transition('true => false', animate('300ms ease-out')),
      transition('false => true', animate('300ms ease-out')),
    ]),
  ],
})
export class StructureEditionSummaryComponent implements OnInit {
  public structure: Structure;
  public structureForm: UntypedFormGroup;
  public members: Owner[];
  public summary: IStructureSummary[] = new FormUtils().structureSummary;
  public structureFormStep = structureFormStep;
  public equipmentEnum = Equipment;
  public trainingCategories: Map<string, Category>;
  public freeWorkshopName: FreeWorkshop = null;

  // Digital learning
  public categories: Category[] = [];
  public mailHref: string;
  // set to true when last updated date of a structure is > 6 months
  public isUpdateStructure = false;
  private isGoToEdit = false;

  /** Modal canExit var */
  public updateModal = false;
  private resolve: Function;

  constructor(
    public profileService: ProfileService,
    private searchService: SearchService,
    private notificationService: NotificationService,
    private route: ActivatedRoute,
    private structureService: StructureService,
    private windowScrollService: WindowScrollService,
    public utils: Utils,
    public router: Router,
  ) {}

  // There is a non-blocking error in console when open edit mode that occurs only in development mode, please refer https://angular.io/errors/NG0100 for more info

  ngOnInit(): void {
    this.windowScrollService.scrolledToBottom = false;
    this.route.data.subscribe((data) => {
      if (data.structure) {
        this.structure = new Structure(data.structure);
        this.structureForm = new FormUtils().createStructureForm(this.structure, true);
        if (this.structure.contactMail) this.mailHref = `mailto:${this.structure.contactMail}`;

        const updatedAt = DateTime.fromISO(this.structure.updatedAt);
        const sixMonthsAgo = DateTime.local().minus({ month: 6 });
        if (updatedAt < sixMonthsAgo) this.isUpdateStructure = true;

        this.structureService.getStructureWithOwners(data.structure._id).subscribe((result) => {
          this.members = result.owners;
          const freeWorkShop = result.structure.categories?.freeWorkShop as unknown as FreeWorkshop;
          if (freeWorkShop) {
            this.freeWorkshopName = FreeWorkshop[freeWorkShop];
          }
        });
      }
      this.initCategories();
    });
  }

  async initCategories(): Promise<void> {
    const categories = await lastValueFrom(this.searchService.getCategories());
    const skillsMap = new Map<string, Category>();
    categories
      .filter((category) => category.id === CategoryEnum.baseSkills || category.id === CategoryEnum.advancedSkills)
      .map((category) => {
        skillsMap.set(category.id, category);
      });

    this.trainingCategories = skillsMap;
  }

  public goBack(): void {
    history.back();
  }

  public goToEdit(step: structureFormStep): void {
    this.isGoToEdit = true;
    this.router.navigate(['/formulaire/structure', this.structure.permalink, structureFormStep[step]]);
  }

  public canExit(): Promise<boolean> {
    if (this.isUpdateStructure && !this.isGoToEdit) {
      return new Promise((resolve) => this.showUpdateModal(resolve));
    }
  }
  private showUpdateModal(resolve: Function): void {
    this.updateModal = true;
    this.resolve = resolve;
  }

  public hasRedirectionAccepted(hasAccept: boolean): void {
    // !hasAccept to invert usual modal confirmation logic (here stay on page if user validates modal, and quit if user cancels)
    this.resolve(!hasAccept);
    this.updateModal = false;
  }

  public isFormValid(): boolean {
    for (const field in this.structure) {
      if (!['_id', 'numero', 'createdAt', 'updatedAt'].includes(field)) {
        if (this.structureForm.get(field)?.valid === false) return false;
      }
    }
    // For structure not updated for 6 months, validation button is disabled until user scrolls to bottom
    if (this.isUpdateStructure && !this.windowScrollService.scrolledToBottom) {
      return false;
    }
    return true;
  }

  public onLastButtonKeydown(event: KeyboardEvent): void {
    switch (event.key) {
      case 'ArrowUp':
      case 'ArrowDown':
      case 'Tab':
        // For keyboard navigation, at the last button of the page, we consider the user reached the bottom of the page
        this.windowScrollService.scrolledToBottom = true;
        break;
    }
  }

  public updateStructureUpdateDate(): void {
    this.structureService.editStructure({}, this.structure._id).subscribe(() => {
      this.notificationService.showSuccess('La structure a bien été mise à jour.');
      this.isUpdateStructure = false;
    });
  }

  public formatAddress(): string {
    let address = '';
    if (this.structure.address.numero) address += `${this.structure.address.numero} `;
    if (this.structure.address.street) address += `${this.structure.address.street}, `;
    if (this.structure.address.commune) address += `${this.structure.address.commune}`;
    return address;
  }

  public getAccessLabel(accessModality: AccessModality): string {
    return this.utils.getAccessLabel(accessModality);
  }

  public containsHours(): boolean {
    for (const day in this.structure.hours) {
      if (this.structure.hours[day].time.length) return true;
    }
    return false;
  }

  public isFieldValid(field: string, theme?: string): boolean {
    if (theme) {
      return this.structureForm.get(theme).get([field]).valid;
    }
    return this.structureForm.get([field]).valid;
  }

  public requiredPhoneOrMailError(): boolean {
    return this.structureForm.hasError('requirePhoneOrMail');
  }

  public hasSocialNetworks(): boolean {
    return this.utils.hasSocialNetwork(this.structure);
  }

  public hasWebsite(): boolean {
    return this.utils.hasWebsite(this.structure);
  }

  public containsDigitalHelp(): boolean {
    return this.structure.categoriesDisplay.onlineProcedures?.length > 0;
  }

  public isBaseSkills(): boolean {
    return this.structure.categories.baseSkills?.length > 0;
  }
  public isAdvancedSkills(): boolean {
    return this.structure.categories.advancedSkills?.length > 0;
  }

  public containsDigitalLearning(): boolean {
    return this.isBaseSkills() || this.isAdvancedSkills();
  }

  public hasWifi(selfServiceMaterial: string[]): boolean {
    return this.utils.hasWifi(selfServiceMaterial);
  }

  public hasEquipments(structure: Structure): boolean {
    return this.utils.hasEquipments(structure.nbComputers, structure.nbPrinters, structure.nbScanners);
  }
  public filterOnlyEquipments(selfServiceMaterial: string[]): string[] {
    return this.utils.filterOnlyEquipments(selfServiceMaterial);
  }
  public getEquipmentsLabelAndValue(equipment: Equipment, number: number): string {
    return this.utils.getEquipmentsLabelAndValue(equipment, number);
  }

  public goToManageMembers(): void {
    this.isGoToEdit = true;
    this.router.navigate([`/profil/gestion-membres/${this.structure.permalink}`]);
  }
}
