import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { Observable, forkJoin } from 'rxjs';
import { tap } from 'rxjs/operators';
import { Structure } from '../../models/structure.model';
import { StructureWithOwners } from '../../models/structureWithOwners.model';
import { User } from '../../models/user.model';
import { NotificationService } from '../../services/notification.service';
import { StructureService } from '../../services/structure.service';
import { ProfileService } from '../services/profile.service';

@Component({
  selector: 'app-structures-management',
  templateUrl: './structures-management.component.html',
  styleUrls: ['./structures-management.component.scss'],
})
export class StructuresManagementComponent implements OnInit {
  public userProfile: User;
  public structures: StructureWithOwners[] = [];
  public selectedStructure: Structure;
  public deleteInProgress: boolean;
  public leaveModalOpened = false;
  public deleteModalOpened = false;
  public cancelDeleteModalOpened = false;

  constructor(
    private notificationService: NotificationService,
    private profileService: ProfileService,
    private router: Router,
    private structureService: StructureService,
  ) {}

  ngOnInit(): void {
    this.getUserAndStructures();
  }

  public goBack(): void {
    history.back();
  }
  public goToFormStructure(): void {
    this.router.navigateByUrl('/formulaire/structure');
  }

  private shouldRedirect(structures: StructureWithOwners[] | Observable<any>[]): void {
    if (!structures.length) {
      this.router.navigate(['/profil']);
    }
  }

  private getStructures(): void {
    const structures$: Observable<any>[] = [];
    this.structures = [];

    this.userProfile.structuresLink.forEach((structureId) => {
      structures$.push(
        this.structureService.getStructureWithOwners(structureId).pipe(
          tap((structure) => {
            this.structures.push(structure);
          }),
        ),
      );
    });

    this.shouldRedirect(structures$);

    forkJoin(structures$).subscribe(() => {
      this.structures.sort((a, b) => a.structure.structureName.localeCompare(b.structure.structureName));
    });
  }

  private getUserAndStructures(): void {
    this.profileService.getProfile().then((profile: User) => {
      this.userProfile = new User(profile);
      this.getStructures();
    });
  }

  public leaveStructure(structure: Structure, shouldExclude: boolean): void {
    this.leaveModalOpened = false;
    if (shouldExclude) {
      this.structureService.removeOwnerFromStructure(this.userProfile._id, structure._id).subscribe(
        () => {
          this.structures = this.structures.filter((obj) => obj.structure._id !== structure._id);
          this.notificationService.showSuccess(`Vous avez bien quitté ${structure.structureName}`);
          this.shouldRedirect(this.structures);
        },
        () => {
          this.notificationService.showError('Une erreur est survenue, veuillez réessayer.');
        },
      );
    }
  }

  public deleteStructure(structure: Structure, shouldDelete: boolean): void {
    this.deleteModalOpened = false;
    if (shouldDelete) {
      this.structureService.delete(structure._id).subscribe((res) => {
        if (!res.toBeDeletedAt) {
          this.notificationService.showSuccess(`La structure ${res.structureName} a bien été supprimée`);
        }
        this.getUserAndStructures();
      });
    }
  }

  public isBeingDeleted(structure: Structure): boolean {
    return Boolean(structure.toBeDeletedAt);
  }

  public cancelDelete(structure: Structure, shouldCancel: boolean): void {
    this.cancelDeleteModalOpened = false;
    if (shouldCancel) {
      this.structureService.cancelDelete(structure._id).subscribe(() => {
        this.getStructures();
      });
    }
  }
}
