import { HttpErrorResponse } from '@angular/common/http';
import { Component, Input } from '@angular/core';
import { AbstractControl, FormControl, FormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { AuthService } from '../services/auth.service';
import { NotificationService } from '../services/notification.service';
import { MustMatch } from '../shared/validator/form';
import { CustomRegExp } from '../utils/CustomRegExp';

@Component({
  selector: 'app-password-form',
  templateUrl: './reset-password.component.html',
  styleUrls: ['./reset-password.component.scss'],
})
export class ResetPasswordComponent {
  @Input({ required: true }) token: string;

  public resetPasswordForm = new FormGroup(
    {
      password: new FormControl('', [Validators.required, Validators.pattern(CustomRegExp.PASSWORD)]),
      confirmPassword: new FormControl('', [Validators.required, Validators.pattern(CustomRegExp.PASSWORD)]),
    },
    { validators: MustMatch('password', 'confirmPassword') },
  );
  public loading: boolean;

  constructor(
    private router: Router,
    private authService: AuthService,
    private notificationService: NotificationService,
  ) {}

  /** Getter for password control */
  get password(): AbstractControl<string, string> {
    return this.resetPasswordForm.get('password');
  }

  /** Getter for confirmPassword control */
  get confirmPassword(): AbstractControl<string, string> {
    return this.resetPasswordForm.get('confirmPassword');
  }

  public checkIfPasswordHasEnoughChar(password: string): boolean {
    return password.length >= 8;
  }

  public checkIfPasswordHasSpecialChar(password: string): boolean {
    return Boolean(RegExp(CustomRegExp.SPECHAR).exec(password));
  }

  public checkIfPasswordHasDigit(password: string): boolean {
    return Boolean(RegExp(CustomRegExp.DIGIT).exec(password));
  }

  public checkIfPasswordHasUpperCase(password: string): boolean {
    return Boolean(RegExp(CustomRegExp.UPPERCASE).exec(password));
  }

  public checkIfPasswordHasLowerCase(password: string): boolean {
    return Boolean(RegExp(CustomRegExp.LOWERCASE).exec(password));
  }

  public passwordIsValid(password: string): boolean {
    return (
      this.checkIfPasswordHasEnoughChar(password) &&
      this.checkIfPasswordHasSpecialChar(password) &&
      this.checkIfPasswordHasDigit(password) &&
      this.checkIfPasswordHasUpperCase(password) &&
      this.checkIfPasswordHasLowerCase(password)
    );
  }

  public onSubmit(): void {
    // stop here if form is invalid
    if (this.resetPasswordForm.invalid) return;

    this.loading = true;
    this.authService.resetPasswordApply(this.token, this.resetPasswordForm.value.password).subscribe({
      next: () => {
        this.notificationService.showSuccess('Votre mot de passe a été réinitialisé avec succès.');
      },
      error: (error: HttpErrorResponse) => {
        this.loading = false;
        if (error.status === 401) {
          this.notificationService.showError('Lien de réinitialisation de mot de passe invalide');
        } else {
          this.notificationService.showError('Échec de la réinitialisation de votre mot de passe');
        }
      },
      complete: () => {
        this.loading = false;
        this.router.navigate(['']);
      },
    });
  }
}
