import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { DateTime } from 'luxon';
import { BehaviorSubject, Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { UserAuth } from '../models/user-auth.model';
import { User } from '../models/user.model';
import { UserNamePipe } from '../shared/pipes';
import { Utils } from '../utils/utils';
@Injectable({
  providedIn: 'root',
})
export class AuthService {
  public userSubject: BehaviorSubject<UserAuth>;
  public user: Observable<UserAuth>;

  constructor(
    private http: HttpClient,
    public utils: Utils,
  ) {
    this.userSubject = new BehaviorSubject<UserAuth>(JSON.parse(localStorage.getItem('user')));
    this.user = this.userSubject.asObservable();
  }

  public get userValue(): UserAuth {
    return this.userSubject.value;
  }

  public get token(): string {
    if (this.userSubject.value) {
      return this.userSubject.value.accessToken;
    }
    return null;
  }

  public logout(): void {
    localStorage.removeItem('user');
    this.userSubject.next(null);
    window.location.replace('/actualites');
  }

  public isLoggedIn(): boolean {
    if (this.userValue) {
      return DateTime.local().setZone('Europe/Paris') < this.getExpiration();
    }
    return false;
  }

  public getUserNameDisplay(): string {
    return new UserNamePipe().transform(this.userValue.name);
  }

  public getUserSurnameDisplay(): string {
    return this.userValue.surname.toUpperCase();
  }

  public getUserFullNameDisplay(): string {
    return `${this.getUserNameDisplay()} ${this.getUserSurnameDisplay()}`;
  }

  public getUserEmailDisplay(): string {
    return `${this.userValue.username}`;
  }

  private getExpiration(): DateTime {
    return DateTime.fromISO(this.userValue.expiresAt, { zone: 'Europe/Paris' });
  }

  public register(user: User): Observable<any> {
    return this.http.post('api/users', user);
  }

  public login(email: string, password: string): Observable<any> {
    return this.http.post<UserAuth>('api/auth/login', { email: email.toLowerCase(), password }).pipe(
      map((user) => {
        // store user details and jwt token in local storage to keep user logged in between page refreshes
        localStorage.setItem('user', JSON.stringify(user));
        this.userSubject.next(user);
        return user;
      }),
    );
  }

  public resendEmail(email: string, password: string): Observable<User> {
    return this.http.post<User>(`api/auth/resendEmail`, { email: email.toLowerCase(), password });
  }

  public verifyUser(userId: string, token: string): Observable<User> {
    return this.http.post<User>(`api/users/verify/${userId}`, null, {
      params: { token },
    });
  }

  public resetPassword(email: string): Observable<any> {
    return this.http.post(`api/users/reset-password`, { email: email.toLowerCase() });
  }

  public checkResetPasswordToken(token: string): Observable<boolean> {
    return this.http.post<boolean>(`api/users/reset-password/check`, { token });
  }

  public resetPasswordApply(token: string, password: string): Observable<any> {
    return this.http.post(`api/users/reset-password/apply`, {
      token,
      password,
    });
  }
}
