import { Component, Input, OnInit } from '@angular/core';
import { AbstractControl, UntypedFormControl, UntypedFormGroup, Validators } from '@angular/forms';
import { Subject } from 'rxjs';
import { Structure } from '../../../models/structure.model';
import { StructureService } from '../../../services/structure.service';

@Component({
  selector: 'app-data-share-consent',
  templateUrl: './data-share-consent.component.html',
  styleUrls: ['./data-share-consent.component.scss'],
})
export class DataShareConsentComponent implements OnInit {
  public consentForm: UntypedFormGroup;
  public isPageValid: boolean;
  public loading = false;
  public submitted = false;
  public eventsSubject: Subject<Object> = new Subject<Object>();
  public allSelected: boolean;

  constructor(private structureService: StructureService) {}

  @Input() public opened = true;
  @Input() public dataConsentPendingStructures: Structure[];

  ngOnInit(): void {
    this.consentForm = new UntypedFormGroup({});
    for (const structure of this.dataConsentPendingStructures) {
      this.consentForm.addControl(
        structure._id,
        new UntypedFormControl(structure.dataShareConsentDate, [Validators.required]),
      );
    }
  }

  ngAfterViewInit(): void {
    // Emit null to eventsSubject so that the radio button is not pre selected when undefined
    this.eventsSubject.next(null);
  }

  public getFormControl(nameControl: string): AbstractControl {
    return this.consentForm.get(nameControl);
  }

  public getPendingStructure(id: string): Structure {
    const result = this.dataConsentPendingStructures.filter((structure) => structure._id === id);
    return result?.[0] || null;
  }

  public onRadioBtnChangeAll(bool: boolean): void {
    for (const structure of this.dataConsentPendingStructures) {
      structure.dataShareConsentDate = bool ? new Date().toString() : null;
      this.getFormControl(structure._id).setValue(bool);
    }
    this.allSelected = bool;
    this.setValidationsForm();
  }

  public onRadioBtnChangeStructure(controlName: string, bool: boolean): void {
    this.getPendingStructure(controlName).dataShareConsentDate = bool ? new Date().toString() : null;
    this.getFormControl(controlName).setValue(bool);

    // select or unselect "all structures" radio button
    let result = null;

    if (this.dataConsentPendingStructures.every((structure) => this.getFormControl(structure._id).value === true)) {
      result = true;
    } else if (
      this.dataConsentPendingStructures.every((structure) => this.getFormControl(structure._id).value === false)
    ) {
      result = false;
    }
    this.eventsSubject.next(result);

    this.setValidationsForm();
  }

  public setValidationsForm(): void {
    let isPageValid = true;
    for (const structure of this.dataConsentPendingStructures) {
      isPageValid = isPageValid && this.getFormControl(structure._id).valid;
    }
    this.isPageValid = isPageValid;
  }

  public onSubmit(): void {
    this.submitted = true;
    this.loading = true;
    for (const structure of this.dataConsentPendingStructures) {
      this.structureService
        .editStructure(
          {
            dataShareConsentDate: structure.dataShareConsentDate,
          },
          structure._id,
        )
        .subscribe(() => {
          this.loading = false;
          this.opened = false;
        });
    }
  }

  public processDataShareConsentDate(structure: Structure): boolean | null {
    if (structure.dataShareConsentDate === undefined) {
      return null;
    } else {
      return structure.dataShareConsentDate !== null;
    }
  }
}
