import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { AbstractControl, UntypedFormArray, UntypedFormControl, UntypedFormGroup, Validators } from '@angular/forms';
import { Day, Time, WeekDayEnum } from '../../../models/week.model';
import { CheckHours } from '../../validator/form';

interface DayHour {
  name: WeekDayEnum;
  hours: {
    start: string;
    end: string;
  }[];
  open: boolean;
}

@Component({
  selector: 'app-hour-picker',
  templateUrl: './hour-picker.component.html',
  styleUrls: ['./hour-picker.component.scss'],
})
export class HourPickerComponent implements OnInit {
  @Input() structureInput: UntypedFormGroup;

  @Output() updateFormError = new EventEmitter<any>();
  @Output() updateForm = new EventEmitter<UntypedFormGroup>();

  public structure = {
    hours: this.initHoursDefault(),
  };

  ngOnInit(): void {
    this.formatHoursForEdition();
  }

  public getStructureControl(nameControl: string): AbstractControl {
    return this.structureInput.get(nameControl);
  }

  private initHoursDefault(): DayHour[] {
    const defaultHours: DayHour[] = [];
    for (const day of Object.values(WeekDayEnum)) {
      defaultHours.push({
        name: day,
        hours: [{ start: '', end: '' }],
        open: false,
      });
    }
    return defaultHours;
  }

  /** Convert data from to component structure */
  private parseFormToHours(day: Day, key: string): void {
    this.structure.hours.forEach((element) => {
      if (element.name.toLowerCase() === key) {
        element.open = day.open;
        element.hours = day.time.map((hour) => ({
          start: hour.opening ?? '',
          end: hour.closing ?? '',
        }));
      }
    });
  }

  private parseToDay(data: DayHour): Day {
    return new Day({
      open: data.open,
      time: data.hours.map(
        (hour) =>
          new Time({
            opening: hour.start,
            closing: hour.end,
          }),
      ),
    });
  }

  private parseHoursToForm(): UntypedFormGroup {
    return new UntypedFormGroup({
      monday: this.createDay(this.parseToDay(this.structure.hours[0])),
      tuesday: this.createDay(this.parseToDay(this.structure.hours[1])),
      wednesday: this.createDay(this.parseToDay(this.structure.hours[2])),
      thursday: this.createDay(this.parseToDay(this.structure.hours[3])),
      friday: this.createDay(this.parseToDay(this.structure.hours[4])),
      saturday: this.createDay(this.parseToDay(this.structure.hours[5])),
      sunday: this.createDay(this.parseToDay(this.structure.hours[6])),
    });
  }

  /**
   * Intégrer les horaires dans les horaires par défaut du composant
   */
  public formatHoursForEdition(): void {
    if (this.structureInput) {
      this.parseFormToHours(this.getStructureControl('monday').value, WeekDayEnum.monday);
      this.parseFormToHours(this.getStructureControl('tuesday').value, WeekDayEnum.tuesday);
      this.parseFormToHours(this.getStructureControl('wednesday').value, WeekDayEnum.wednesday);
      this.parseFormToHours(this.getStructureControl('thursday').value, WeekDayEnum.thursday);
      this.parseFormToHours(this.getStructureControl('friday').value, WeekDayEnum.friday);
      this.parseFormToHours(this.getStructureControl('saturday').value, WeekDayEnum.saturday);
      this.parseFormToHours(this.getStructureControl('sunday').value, WeekDayEnum.sunday);
    }
  }

  public toggleOpenDay(day: DayHour, checked: boolean): void {
    day.open = checked;

    if (!checked) {
      day.hours = [];
    } else {
      this.addHours(day);
    }
    this.submitForm();
  }

  /** Ajouter une ligne d'horaires à un jour */
  public addHours(day: DayHour): void {
    if (day.hours.length >= 5) {
      return;
    }

    day.hours.push({
      start: '',
      end: '',
    });
    this.submitForm();
  }

  /** Supprimer la dernière ligne d'horaires d'un jour */
  public removeHours(day: DayHour, index: number): void {
    if (index > -1) {
      day.hours.splice(index, 1);
      this.submitForm();
    }
  }

  /** Vérifier que le format des horaires est correct */
  public checkHoursValid(): boolean {
    for (const day of this.structure.hours) {
      if (!day.open) continue;
      for (const hour of day.hours) {
        if (this.getStatus(hour.start, hour.end) !== 'success') {
          return false;
        }
      }
    }
    return true;
  }

  public submitForm(): void {
    if (this.checkHoursValid()) {
      this.updateForm.emit(this.parseHoursToForm());
    } else {
      this.updateFormError.emit();
    }
  }

  private createDay(day: Day): UntypedFormGroup {
    return new UntypedFormGroup({
      open: new UntypedFormControl(day.open, Validators.required),
      time: new UntypedFormArray(day.time.map((oneTime) => this.createTime(oneTime))),
    });
  }

  private createTime(time: Time): UntypedFormGroup {
    return new UntypedFormGroup({
      opening: new UntypedFormControl(time.opening, Validators.required),
      closing: new UntypedFormControl(time.closing, [Validators.required, CheckHours(time.opening)]),
    });
  }

  getStatus(start: string, end?: string): 'error' | 'success' {
    if (end === undefined) {
      return start !== '' ? 'success' : 'error';
    }
    return end && end > start ? 'success' : 'error';
  }

  getStatusText(start: string, end?: string): string {
    return 'Horaire ' + (this.getStatus(start, end) === 'error' ? 'invalide' : 'valide');
  }
}
