import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { CustomRegExp } from 'src/app/utils/CustomRegExp';
import { Utils } from '../../../utils/utils';

@Component({
  selector: 'app-input',
  templateUrl: './input.component.html',
  styleUrls: ['./input.component.scss'],
})
export class InputComponent implements OnInit {
  /** HTML id associated with for */
  @Input() id: string;

  @Input() type: 'text' | 'password' | 'time' | 'email' | 'tel' = 'text';

  @Input() disabled = false;

  @Input() readonly = false;

  /** If set to 'on', enables autocomplete of the input DOM */
  @Input() autocomplete: 'on' | 'off' = 'off';

  /** Label for the input */
  @Input() label?: string;

  /** Affect the height of the input */
  @Input() size?: 'small' | 'large';

  /** If true, removes the max-width of the component to take 100% width */
  @Input() wide?: boolean;

  /** Additional label description */
  @Input() description?: string;

  /** What additional data should be displayed ? */
  @Input() status?: 'error' | 'success' | 'info' | 'warning';

  /** Additional text to display */
  @Input() statusText?: string;

  /** Text for placeholder */
  @Input() placeholder?: string = '';

  @Input() value: string;

  @Input() required = false;

  /** Additional sup information next to the label */
  @Input() sup = '';

  /** Determines if the parent component controls the status and statusText */
  @Input() externalStatusControl = false;

  /** Triggers when input changes */
  @Output() valueChange = new EventEmitter<string>();

  @Output() finishedEditing = new EventEmitter<string>();

  constructor(private utils: Utils) {}

  public hasIconInField = false;
  private example = '';

  public get classes(): string {
    return [this.size, this.status, this.wide ? 'wide' : ''].join(' ');
  }

  ngOnInit(): void {
    if (this.type === 'password' || this.readonly) this.hasIconInField = true;
    if (this.required) this.sup = '*';
  }

  public onChange(): void {
    if (!this.externalStatusControl) {
      if (this.type === 'text' && this.isInvalidText()) {
        this.setStatusError(this.getStatusText());
      } else if (this.type === 'email' && !Boolean(this.value.match(CustomRegExp.EMAIL))) {
        this.setStatusError(this.getExampleText());
      } else if (this.type === 'tel' && !Boolean(this.value.match(CustomRegExp.PHONE))) {
        this.setStatusError(this.getExampleText());
      } else {
        this.setStatusSuccess();
      }
    }
    if (this.type === 'tel') {
      this.value = this.utils.modifyPhoneValue(this.value);
    }
    this.valueChange.emit(this.value);
  }

  public onFinishedEditing(): void {
    this.finishedEditing.emit(this.value);
  }

  public toggleShowPassword(): void {
    this.type = this.type === 'password' ? 'text' : 'password';
  }

  getStatusText(): string {
    if (this.externalStatusControl && this.statusText) {
      return this.statusText;
    }
    if (this.value === '') {
      return '';
    }
    if (this.status === 'error') {
      this.example = this.getExampleText();
      return `${this.label} invalide. ${this.example}`;
    }
    if (this.status === 'success') {
      return `${this.label} valide`;
    }
    return '';
  }

  private isInvalidText(): boolean {
    return (this.id.startsWith('name') || this.id.startsWith('surname')) && !CustomRegExp.NAME.test(this.value);
  }

  private setStatusError(message: string): void {
    this.status = 'error';
    this.statusText = message;
  }

  private setStatusSuccess(): void {
    this.status = 'success';
    this.statusText = '';
  }

  private getExampleText(): string {
    switch (true) {
      case this.type === 'email':
        return 'Exemple: ml@gmail.com';
      case this.type === 'tel':
        return 'Exemple: 06 11 22 33 44';
      case this.id.startsWith('name'):
        return this.getNameExampleText();
      case this.id.startsWith('surname'):
        return 'Exemple: Bocuse';
      default:
        return '';
    }
  }

  private getNameExampleText(): string {
    if (this.id.startsWith('name')) {
      if (this.label === 'Prénom') {
        return 'Exemple : Paul';
      } else {
        return 'Exemple : Paul Bocuse';
      }
    }
  }
}
