import { animate, style, transition, trigger } from '@angular/animations';
import { Component, ElementRef, EventEmitter, Input, OnInit, Output, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import * as _ from 'lodash';
import { Owner } from '../../../models/owner.model';
import { Structure } from '../../../models/structure.model';
import { User } from '../../../models/user.model';
import { ProfileService } from '../../../profile/services/profile.service';
import { AuthService } from '../../../services/auth.service';
import { StructureService } from '../../../services/structure.service';
import { UserService } from '../../../services/user.service';
import { Utils } from '../../../utils/utils';
import { AccessModality } from '../../enum/access-modality.enum';
import { Equipment } from '../../enum/equipment.enum';
import { Category } from '../../models/category.model';
import { SearchService } from '../../services/search.service';

@Component({
  selector: 'app-structure-details',
  templateUrl: './structure-details.component.html',
  styleUrls: ['./structure-details.component.scss'],
  animations: [
    trigger('slideInOut', [
      transition(':enter', [style({ left: '-600px' }), animate('200ms ease-in', style({ left: '0' }))]),
      transition(':leave', [animate('200ms ease-in', style({ left: '-600px' }))]),
    ]),
    trigger('fadeInOut', [
      transition(':enter', [
        style({ backgroundColor: 'rgb(00, 00, 00, 0)' }),
        animate('200ms ease-in', style({ backgroundColor: 'rgb(00, 00, 00, 0.6)' })),
      ]),
      transition(':leave', [animate('200ms ease-in', style({ backgroundColor: 'rgb(00, 00, 00, 0)' }))]),
    ]),
  ],
})
export class StructureDetailsComponent implements OnInit {
  @Input() public structure: Structure;
  @Output() public closeDetails: EventEmitter<boolean> = new EventEmitter<boolean>();
  public accessModality = AccessModality;
  public categories: Category[] = [];
  public printMode = false;
  public isLoading = true;
  public currentProfile: User = null;
  public deleteModalOpened = false;
  public structureErrorModalOpened = false;
  public claimModalOpened: boolean;
  public joinModalOpened = false;
  public pendingModalOpened = false;
  public structureAdmins: Owner[] = [];
  public membersWithJobWithPO: Owner[] = [];
  public fullScreen = false;
  public showButtons = true;
  public Equipment = Equipment;
  public isInStructure = false;
  public hasTclInfo = true;

  @ViewChild('closeButton', { read: ElementRef }) closeButtonElement: ElementRef;

  constructor(
    private searchService: SearchService,
    private structureService: StructureService,
    public profileService: ProfileService,
    private authService: AuthService,
    private route: ActivatedRoute,
    private router: Router,
    private utils: Utils,
    private usersService: UserService,
  ) {}

  async ngOnInit(): Promise<void> {
    this.route.data.subscribe((data) => {
      if (data.structure) {
        this.structure = new Structure(data.structure);
        this.profileIsLinkedToStructure(this.structure._id);
        this.isLoading = true;
        this.initForm();
      } else {
        this.structure = null;
      }
      if (data.fullScreen) {
        this.fullScreen = true;
      }
      this.showButtons = !history.state.isOrientation;
    });
  }

  private async initForm(): Promise<void> {
    if (this.userIsLoggedIn()) {
      this.currentProfile = await this.profileService.getProfile();
      this.structureService.getStructureWithOwners(this.structure._id).subscribe((res) => {
        this.structureAdmins = res.owners;
        this.membersWithJobWithPO = res.owners.filter((owner) => owner.job?.hasPersonalOffer);
      });
    }

    this.searchService.getCategories().subscribe((categories) => {
      this.categories = categories;
      this.setServiceCategories();
      this.isLoading = false;
      setTimeout(() => {
        this.closeButtonElement.nativeElement.children[0].focus();
      });
    });
  }

  public async profileIsLinkedToStructure(id: string): Promise<void> {
    this.isInStructure = await this.profileService.isLinkedToStructure(id);
  }

  public userIsLoggedIn(): boolean {
    return this.authService.isLoggedIn();
  }

  public getEquipmentsLabel(equipment: Equipment): string {
    switch (equipment) {
      case Equipment.wifi:
        return 'Wifi en accès libre';
      case Equipment.printer:
        return this.structure.nbPrinters > 1 ? 'Imprimantes' : 'Imprimante';
      case Equipment.computer:
        return this.structure.nbComputers > 1 ? 'Ordinateurs' : 'Ordinateur';
      case Equipment.scanner:
        return this.structure.nbScanners > 1 ? 'Scanners' : 'Scanner';
      default:
        return null;
    }
  }

  public close(): void {
    this.router.navigate([], {
      queryParams: {
        structure: null,
      },
      queryParamsHandling: 'merge',
    });
  }

  public print(): void {
    setTimeout(() => {
      window.print();
    }, 1500);
  }

  public toggleDeleteModal(): void {
    this.deleteModalOpened = !this.deleteModalOpened;
  }

  public toggleClaimModal(): void {
    this.claimModalOpened = !this.claimModalOpened;
  }

  public toggleJoinModal(): void {
    this.joinModalOpened = !this.joinModalOpened;
  }

  public togglePendingModal(): void {
    this.pendingModalOpened = !this.pendingModalOpened;
  }

  public handleJoin(): void {
    if (!this.userIsLoggedIn()) {
      this.router.navigate(['connexion'], {
        queryParams: { returnUrl: `acteurs?structure=${this.structure.permalink}` },
      });
      return;
    }

    if (this.profileService.isPendingLinkedToStructure(this.structure._id)) {
      this.togglePendingModal();
    } else if (this.structure.hasNoUserDN) {
      this.toggleClaimModal();
    } else {
      this.toggleJoinModal();
    }
  }

  public handleModify(): void {
    this.router.navigate(['profil', 'edition-structure', this.structure.permalink]);
  }

  public deleteStructure(shouldDelete: boolean): void {
    this.toggleDeleteModal();
    if (shouldDelete) {
      this.structureService.delete(this.structure._id).subscribe(() => {
        this.reload();
      });
    }
  }

  /** full reload to avoid strange map errors loop when reloading via angular router */
  private reload(): void {
    window.location.replace('/acteurs');
  }

  public claimStructure(shouldClaim: boolean): void {
    this.toggleClaimModal();
    if (shouldClaim) {
      this.structureService
        .claimStructureWithAccount(this.structure._id, this.authService.userValue.username)
        .subscribe();
      this.router.navigate(['join-request', this.structure.permalink], {
        state: { isClaimed: !this.structure.hasNoUserDN },
      });
    }
  }

  public joinStructure(shouldJoin: boolean): void {
    this.toggleJoinModal();
    if (shouldJoin) {
      this.usersService.joinStructure(this.structure._id, this.authService.userValue.username).subscribe();
      this.router.navigate(['join-request', this.structure.permalink], {
        state: { isClaimed: !this.structure.hasNoUserDN },
      });
    }
  }

  /**
   * Map categories ids to their real names
   */
  public setServiceCategories(): void {
    // Set category names in this.structure.categoriesDisplay
    this.utils.setServiceCategories(this.categories, this.structure);

    // Merge the structure offer with the personal offers of all members of the structure
    this.structure.personalOffers.forEach((personalOffer) => {
      const personalOfferDisplay = this.utils.setServiceCategories(this.categories, personalOffer);

      // use lodash _.union fonction to concat array without duplicates
      this.structure.categoriesDisplay.onlineProcedures = _.union(
        this.structure.categoriesDisplay.onlineProcedures,
        personalOfferDisplay.categoriesDisplay.onlineProcedures,
      );
      this.structure.categoriesDisplay.baseSkills = _.union(
        this.structure.categoriesDisplay.baseSkills,
        personalOfferDisplay.categoriesDisplay.baseSkills,
      );
      this.structure.categoriesDisplay.advancedSkills = _.union(
        this.structure.categoriesDisplay.advancedSkills,
        personalOfferDisplay.categoriesDisplay.advancedSkills,
      );
    });
  }

  public hasBaseSkills(): boolean {
    return this.structure.categoriesDisplay.baseSkills?.length > 0;
  }
  public hasAdvancedSkills(): boolean {
    return this.structure.categoriesDisplay.advancedSkills?.length > 0;
  }

  public displayModalError(): void {
    this.structureErrorModalOpened = !this.structureErrorModalOpened;
  }

  public sendErrorEmail(shouldSend: boolean, content?: string): void {
    this.displayModalError();
    if (shouldSend && content) {
      this.structureService.sendMailOnStructureError(this.structure._id, content).subscribe();
    }
  }

  public goToWebsite(): void {
    let url = this.structure.website;
    if (!url.startsWith('http')) {
      url = 'https://' + url;
    }
    window.open(url, '_blank');
  }
  public goToOrientation(): void {
    this.router.navigateByUrl('/orientation', { state: { rdvStructure: this.structure } });
  }

  public displayTrainings(): boolean {
    if (this.structure.categoriesDisplay === undefined) return false;

    return (
      (this.structure.categoriesDisplay.age && this.structure.categoriesDisplay.age.length !== 0) ||
      (this.structure.categoriesDisplay.languageAndIlliteracy &&
        this.structure.categoriesDisplay.languageAndIlliteracy.length !== 0) ||
      (this.structure.categoriesDisplay.handicaps && this.structure.categoriesDisplay.handicaps.length !== 0) ||
      (this.structure.categoriesDisplay.publicOther && this.structure.categoriesDisplay.publicOther.length !== 0)
    );
  }

  public containsHours(): boolean {
    for (const day in this.structure.hours) {
      if (this.structure.hours[day].time.length) return true;
    }
    return false;
  }
}
