import { Component, ElementRef, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Utils } from '../../../utils/utils';
import { Theme } from '../../enum/themes.enum';
import { TypeModal } from '../../enum/typeModal.enum';
import { Category } from '../../models/category.model';
import { Filter } from '../../models/filter.model';
import { Module } from '../../models/module.model';
import { SearchService } from '../../services/search.service';

@Component({
  selector: 'app-structure-list-search',
  templateUrl: './structure-list-search.component.html',
  styleUrls: ['./structure-list-search.component.scss'],
})
export class StructureListSearchComponent implements OnInit {
  @Output() searchEvent = new EventEmitter();
  @Input() filters: Filter[] = [];
  public utils = new Utils();

  public searchInput = '';
  public modalTypeOpened: TypeModal;
  // Checkbox variable
  public checkedModulesFilter: Module[] = [];

  public numberTrainingChecked = 0;
  public numberAccompanimentChecked = 0;
  public numberPublicChecked = 0;
  public numberEquipmentChecked = 0;
  public numberMoreFiltersChecked = 0;
  public numberFreeWorkShopsChecked = 0;
  // Modal categories
  public categoriesTraining: Category[] = [];
  public categoriesAccompaniment: Category[] = [];
  public categoriesPublic: Category[] = [];
  public categoriesEquipment: Category[] = [];
  public categoriesMoreFilters: Category[] = [];
  public categoriesFreeWorkshops: Category[] = [];
  public prmAdded = false;

  public hasUrlParams = false;
  public keyboardEvent = false;

  constructor(
    public searchService: SearchService,
    private activatedRoute: ActivatedRoute,
    private router: Router,
    private elementRef: ElementRef,
  ) {}
  ngOnInit(): void {
    this.searchInput = this.activatedRoute.snapshot.queryParamMap.get('query');
    this.searchService.getCategories().subscribe((categories: Category[]) => {
      // manually add PMR module for More Filters modal
      if (!this.prmAdded) {
        const index = categories.findIndex((element) => element.id === 'accessModality');
        if (index > -1) {
          categories[index].modules.push(new Module('True', 'Accessible PMR', 'Accessible PMR'));
          this.prmAdded = true;
        }
      }
      // Get URL parameters if they exist and make them filters
      this.urlParamsToFilters(categories);
      // Convert input filters to Module
      this.checkedModulesFilter = this.utils.convertFiltersToModule(this.filters);
      // Get the categories for each modal type
      this.getModalCategories(categories);

      this.fetchResults(this.checkedModulesFilter);
    });
  }

  public convertModulesToFilters(modules: Module[], term?: string): Filter[] {
    const filters: Filter[] = [];
    if (term) {
      filters.push(new Filter('query', term));
    }
    // Add checked box filter
    modules.forEach((cm) => {
      filters.push(new Filter(cm.name, cm.id, cm.displayText, ['ctm', 'freeWorkShop'].includes(cm.name)));
    });
    return filters;
  }

  // Accessor to template angular.
  public get TypeModal(): typeof TypeModal {
    return TypeModal;
  }

  public applyFilter(): void {
    const filters = this.convertModulesToFilters(this.checkedModulesFilter, this.searchInput);
    this.updateUrlParams(filters);
    this.countCheckFiltersOnModules(this.checkedModulesFilter);
    this.searchEvent.emit(filters);
  }

  // Update url with new params added or removed
  public updateUrlParams(filters: Filter[]): void {
    // No url update if the page is displaying structure details, because it must keep the url with the structure id, neither for orientation
    if (
      (this.activatedRoute.snapshot.queryParams.structure && filters.length === 0) ||
      this.router.url.startsWith('/orientation')
    ) {
      return;
    }

    const queryString = [];
    filters.forEach((elt) => {
      if (queryString[elt.name]) {
        queryString[elt.name] = queryString[elt.name] + ',' + elt.value;
      } else {
        queryString[elt.name] = elt.value;
      }
    });

    this.checkedModulesFilter = this.utils.convertFiltersToModule(filters);

    this.router.navigate(['./'], {
      relativeTo: this.activatedRoute,
      queryParams: queryString,
    });
  }

  public fetchResults(checkedModules: Module[]): void {
    // Check if some modules is checked in filters
    if (this.checkedModulesFilter !== checkedModules) {
      this.countCheckFiltersOnModules(checkedModules);
    }
    // Store checked modules
    this.checkedModulesFilter = checkedModules;
    // Close modal after receive filters from her.
    this.closeModal();
    this.applyFilter();
  }

  // Check if some modules is checked on filter and store number of modules checked
  public countCheckFiltersOnModules(checkedModules: Module[]): void {
    this.numberAccompanimentChecked = checkedModules.filter((module) => module.name === 'onlineProcedures').length;
    this.numberTrainingChecked = checkedModules.filter(
      (module) =>
        module.name === 'onlineProcedures' || module.name === 'baseSkills' || module.name === 'advancedSkills',
    ).length;
    this.numberPublicChecked = checkedModules.filter(
      (module) =>
        module.name === 'age' ||
        module.name === 'languageAndIlliteracy' ||
        module.name === 'handicaps' ||
        module.name === 'genre',
    ).length;
    this.numberEquipmentChecked = checkedModules.filter(
      (module) => module.name === 'solidarityMaterial' || module.name === 'selfServiceMaterial',
    ).length;
    this.numberMoreFiltersChecked = checkedModules.filter(
      (module) => module.name === 'labelsQualifications' || module.name === 'accessModality' || module.name === 'ctm',
    ).length;
    this.numberFreeWorkShopsChecked = checkedModules.filter((module) => module.name === 'freeWorkShop').length;
  }

  public getModalCategory(): Category[] {
    switch (this.modalTypeOpened) {
      case TypeModal.accompaniment:
        return this.categoriesAccompaniment;
      case TypeModal.training:
        return this.categoriesTraining;

      case TypeModal.public:
        return this.categoriesPublic;

      case TypeModal.equipments:
        return this.categoriesEquipment;

      case TypeModal.moreFilters:
        return this.categoriesMoreFilters;

      case TypeModal.freeWorkShop:
        return this.categoriesFreeWorkshops;

      default:
        throw new Error('Modal type not handle');
    }
  }

  // Open the modal and display the list according to the right filter button
  public openModal(modalType: TypeModal): void {
    // if modal already opened, reset type
    if (this.modalTypeOpened === modalType) {
      this.closeModal();
    } else if (this.modalTypeOpened !== modalType) {
      this.modalTypeOpened = modalType;

      // Accessibility: when navigating with keyboard and opening a filter modal, send focus to the first focusable element of the opened modal
      if (this.keyboardEvent) {
        setTimeout(() => {
          this.utils.setFocusOnElement(this.elementRef, `.modalContent .collapse-header, .modalContent input`);
        }, 0);
      }
    }
  }

  // When filters and their modal are in the same component, we can remove onKeyboardNavOnFilters, setFocusOnOpenedModal, and setFocusOnFilters.
  // because the focus will then flow normally between the filter and the modal
  public onKeyboardNavOnFilters(event: KeyboardEvent): void {
    switch (event.key) {
      case 'ArrowUp':
      case 'ArrowDown':
      case 'Tab':
        this.keyboardEvent = true;
        break;
    }
  }

  public closeModal(): void {
    // Accessibility: when navigating with keyboard and closing a filter modal, send focus back to filters
    if (this.keyboardEvent) {
      this.utils.setFocusOnElement(this.elementRef, '#modal' + this.modalTypeOpened + ' button:first-of-type');
    }
    this.modalTypeOpened = undefined;
  }

  // Management of the checkbox event (Check / Uncheck)
  public externalCheckboxCheck(module: Module): void {
    this.closeModal();
    // Check if the module is present in the list
    const { id, name, displayText } = module;
    const index = this.checkedModulesFilter.findIndex((m) => m.id === id && m.name === name);
    if (index > -1) {
      // If the module is present in the list, we remove it
      this.checkedModulesFilter.splice(index, 1);
      this.countCheckFiltersOnModules(this.checkedModulesFilter);
    } else {
      // If the module is not present in the list, we add it
      this.checkedModulesFilter.push(new Module(id, name, displayText));
      this.numberMoreFiltersChecked++;
    }
    this.applyFilter();
  }

  // Get URL parameters if they exist and make them filters
  private urlParamsToFilters(categories: Category[]): void {
    // array of categories names for comparison with url parameters
    const catNames = [];
    categories.forEach((cat) => {
      catNames.push(cat.id);
    });
    catNames.push('pmrAccess');

    // get URL params if they are present and match a category and make them filters
    this.hasUrlParams = Object.keys(this.activatedRoute.snapshot.queryParams).length > 0;
    if (this.hasUrlParams) {
      const filters: Filter[] = [];
      const filterSet = new Set<string>();

      this.activatedRoute.queryParamMap.subscribe((params) => {
        const paramKeys = { ...params.keys };

        for (const key in paramKeys) {
          if (paramKeys[key] !== 'query' && catNames.includes(paramKeys[key])) {
            const categoryId = paramKeys[key];
            params
              .get(categoryId)
              .split(',')
              .forEach((moduleId) => {
                const filterKey = `${categoryId}_${moduleId}`; // create unique key to compare if filter already exists (duplicate values in url)
                if (!filterSet.has(filterKey)) {
                  filterSet.add(filterKey);
                  const displayText = categories
                    .find((cat) => cat.id === categoryId)
                    .modules.find((mod) => mod.id === moduleId).displayText;
                  filters.push(new Filter(categoryId, moduleId, displayText));
                }
              });
          }
        }

        this.filters = filters;
      });
    }
  }

  // get categories for each modal type
  private getModalCategories(categories: Category[]): void {
    categories.forEach((category) => {
      switch (category.theme) {
        case Theme.skills:
          this.categoriesTraining.push(category);
          // no sort here because we want trainings to be ordered like they are in the database
          break;
        case Theme.materialAndWifi:
          this.categoriesEquipment.push(category);
          this.categoriesEquipment.sort((a, b) => a.name.localeCompare(b.name));
          break;
        case Theme.public:
          this.categoriesPublic.push(category);
          this.categoriesPublic.sort((a, b) => a.name.localeCompare(b.name));
          break;
        case Theme.freeWorkShop:
          this.categoriesFreeWorkshops.push(category);
          this.categoriesFreeWorkshops.sort((a, b) => a.name.localeCompare(b.name));
          break;
        case Theme.noModal:
          break;
        default:
          this.categoriesMoreFilters.push(category);
          this.categoriesMoreFilters.sort((a, b) => a.name.localeCompare(b.name));
          break;
      }
    });
  }

  public resetFilters(): void {
    this.searchInput = '';
    this.checkedModulesFilter = [];
    this.numberTrainingChecked = 0;
    this.numberAccompanimentChecked = 0;
    this.numberPublicChecked = 0;
    this.numberEquipmentChecked = 0;
    this.numberMoreFiltersChecked = 0;
    this.numberFreeWorkShopsChecked = 0;
    const filters = this.convertModulesToFilters(this.checkedModulesFilter, this.searchInput);
    this.updateUrlParams(filters);
    this.searchEvent.emit(filters);
  }
  public removeFilter(module: Module): void {
    const index = this.checkedModulesFilter.findIndex((m: Module) => m.id === module.id);
    this.checkedModulesFilter.splice(index, 1);
    const filters = this.convertModulesToFilters(this.checkedModulesFilter, this.searchInput);
    this.countCheckFiltersOnModules(this.checkedModulesFilter);
    this.updateUrlParams(filters);
    this.searchEvent.emit(filters);
  }
}
