import { FormControl, FormGroup, UntypedFormControl, UntypedFormGroup, Validators } from '@angular/forms';
import { structureFormStep } from '../form/form-view/structure-form/structureFormStep.enum';
import {
  AccompanimentTypes,
  AppointmentSteps,
  CommonSteps,
  NeedsTypes,
  OnlineMediationSteps,
  StructuresListSteps,
} from '../form/orientation-form-view/enums/orientation.enums';
import { AppointmentForm } from '../form/orientation-form-view/interfaces/appointmentForm.interface';
import { FiltersForm } from '../form/orientation-form-view/interfaces/filtersForm.interface';
import { OnlineDemarchesForm } from '../form/orientation-form-view/interfaces/onlineDemarchesForm.interface';
import { StructureOrientationForm } from '../form/orientation-form-view/interfaces/structureOrientationForm.interface';
import { StructureOrientator } from '../form/orientation-form-view/interfaces/structureOrientator.interface';
import { PreferredLanguages } from '../form/orientation-form-view/online-demarch/mediation-language-selection/mediation-language-selection.component';
import { PersonalOffer } from '../models/personalOffer.model';
import { OrientationService } from '../services/orientation.service';
import { Filter } from '../structure-list/models/filter.model';
import { Module } from '../structure-list/models/module.model';
import { CustomRegExp } from './CustomRegExp';

export interface IStructureSummary {
  step: structureFormStep;
  name: string;
  inputs: string[] | null;
  hideForEdit?: boolean;
}
export interface INeedItem {
  title: string;
  hint: string;
  key: NeedsTypes | AccompanimentTypes;
  icon?: string;
}
export class OrientationUtils {
  public createOnlineDemarchesForm(): FormGroup<OnlineDemarchesForm> {
    return new FormGroup<OnlineDemarchesForm>({
      onlineDemarcheType: new FormControl<Module[]>([], Validators.required),
      accompanimentType: new UntypedFormControl(null, Validators.required),
    });
  }

  public createFiltersForm(): FormGroup<FiltersForm> {
    return new FormGroup<FiltersForm>({
      filters: new FormControl([], Validators.required),
    });
  }

  public createOnlineMediationForm(
    onlineDemarcheType: Module[],
    orientatorForm?: FormGroup<StructureOrientator>,
  ): UntypedFormGroup {
    return new UntypedFormGroup({
      filters: new FormControl<Module[]>(onlineDemarcheType, Validators.required),
      name: new UntypedFormControl('', [Validators.required, Validators.pattern(CustomRegExp.TEXT_WITHOUT_NUMBER)]),
      surname: new UntypedFormControl('', [Validators.required, Validators.pattern(CustomRegExp.TEXT_WITHOUT_NUMBER)]),
      phone: new UntypedFormControl('', [Validators.required, Validators.pattern(CustomRegExp.PHONE)]),
      dateSlot: new UntypedFormControl(null, Validators.required),
      preferredLanguage: new UntypedFormControl(PreferredLanguages.french, Validators.required),
      comments: new UntypedFormControl('', Validators.required),
      structureOrientator: orientatorForm || this.createStructureOrientatorForm(),
    });
  }

  public createStructuresListForm(
    filters: Module[] = [],
    orientatorForm?: FormGroup<StructureOrientator>,
  ): FormGroup<StructureOrientationForm> {
    return new FormGroup<StructureOrientationForm>({
      filters: new FormControl<Module[]>(filters, Validators.required),
      pmrAccess: new FormControl(null, Validators.required),
      address: new FormControl(''),
      structureChoice: new FormControl([], Validators.required),
      name: new FormControl('', [Validators.required, Validators.pattern(CustomRegExp.TEXT_WITHOUT_NUMBER)]),
      surname: new FormControl('', [Validators.required, Validators.pattern(CustomRegExp.TEXT_WITHOUT_NUMBER)]),
      comments: new FormControl('', [Validators.required]),
      structureOrientator: orientatorForm || this.createStructureOrientatorForm(),
    });
  }

  public createStructureOrientatorForm(orientator?: {
    structureName: string;
    structureMail: string;
    structurePhone: string;
  }): FormGroup<StructureOrientator> {
    return new FormGroup<StructureOrientator>({
      structureName: new FormControl(orientator?.structureName || '', Validators.required),
      structureMail: new FormControl(orientator?.structureMail || '', [
        Validators.required,
        Validators.pattern(CustomRegExp.EMAIL),
      ]),
      structurePhone: new FormControl(orientator?.structurePhone || '', [
        Validators.required,
        Validators.pattern(CustomRegExp.PHONE),
      ]),
    });
  }

  private isStructureOrientatorFormValid(orientatorForm: FormGroup): boolean {
    const form = orientatorForm.controls;
    return (
      form.structureName?.valid &&
      (form.structurePhone?.valid || form.structureMail?.valid) &&
      (!form.structurePhone?.value || form.structurePhone?.valid) &&
      (!form.structureMail?.value || form.structureMail?.valid)
    );
  }

  public createAppointmentForm(
    onlineDemarcheType,
    orientatorForm?: FormGroup<StructureOrientator>,
  ): FormGroup<AppointmentForm> {
    return new FormGroup<AppointmentForm>({
      structureOrientator: orientatorForm || this.createStructureOrientatorForm(),
      structureRDV: new FormControl('', Validators.required),
      pmrAccess: new FormControl(null, Validators.required),
      address: new FormControl(''),
      onlineDemarcheType: new FormControl(onlineDemarcheType),
      socialWorkerId: new FormControl('', Validators.required),
      details: new FormControl(''),
      name: new FormControl('', [Validators.required, Validators.pattern(CustomRegExp.TEXT_WITHOUT_NUMBER)]),
      surname: new FormControl('', [Validators.required, Validators.pattern(CustomRegExp.TEXT_WITHOUT_NUMBER)]),
      email: new UntypedFormControl('', [Validators.required, Validators.pattern(CustomRegExp.EMAIL)]),
      phone: new UntypedFormControl('', [Validators.required, Validators.pattern(CustomRegExp.PHONE)]),
    });
  }

  public setValidationsOnlineDemarchesForm(
    pagesValidation: any[],
    filtersForm: FormGroup<FiltersForm>,
    form: UntypedFormGroup,
    updatePageValid: (isValid: boolean) => void,
    step: CommonSteps,
  ): void {
    pagesValidation[CommonSteps.filtersChoice] = {
      valid: filtersForm.get('filters').value.length > 0,
    };
    pagesValidation[CommonSteps.accompanimentType] = {
      valid: form.get('accompanimentType')?.value !== null,
    };
    updatePageValid(pagesValidation[step].valid);
  }

  public setValidationsOnlineMediationForm(
    pagesValidation: any[],
    form: UntypedFormGroup,
    updatePageValid: (isValid: boolean) => void,
    step: OnlineMediationSteps,
  ): void {
    pagesValidation[OnlineMediationSteps.infoScreen] = { valid: true };
    pagesValidation[OnlineMediationSteps.mediationBeneficiaryInfo] = {
      valid: form.get('name').valid && form.get('surname').valid && form.get('phone').valid,
    };
    pagesValidation[OnlineMediationSteps.mediationHoursSelection] = {
      valid: form.get('dateSlot').value !== null,
    };
    pagesValidation[OnlineMediationSteps.mediationLanguageSelection] = { valid: true };
    pagesValidation[OnlineMediationSteps.comments] = { valid: true };
    pagesValidation[OnlineMediationSteps.structureOrientator] = {
      valid: this.isStructureOrientatorFormValid(form.get('structureOrientator') as FormGroup),
    };
    pagesValidation[OnlineMediationSteps.orientationRecap] = { valid: true };
    pagesValidation[OnlineMediationSteps.rdvEnd] = { valid: true };
    pagesValidation[OnlineMediationSteps.orientationPrint] = { valid: true };
    updatePageValid(pagesValidation[step].valid);
  }

  public setValidationsStructuresForm(
    pagesValidation: any[],
    form: FormGroup<StructureOrientationForm>,
    updatePageValid: (isValid: boolean) => void,
    step: StructuresListSteps,
  ): void {
    pagesValidation[StructuresListSteps.pmrAccess] = {
      valid: form.get('pmrAccess').valid,
    };
    pagesValidation[StructuresListSteps.address] = { valid: true };
    pagesValidation[StructuresListSteps.structureChoice] = {
      valid: form.get('structureChoice').valid,
    };
    pagesValidation[StructuresListSteps.mediationBeneficiaryInfo] = {
      valid: form.get('name').valid && form.get('surname').valid,
    };
    pagesValidation[StructuresListSteps.comments] = { valid: true };
    pagesValidation[StructuresListSteps.structureOrientator] = {
      valid: this.isStructureOrientatorFormValid(form.get('structureOrientator') as FormGroup),
    };
    pagesValidation[StructuresListSteps.orientationRecap] = { valid: true };
    updatePageValid(pagesValidation[step].valid);
  }

  public setValidationsAppointmentForm(
    pagesValidation: any[],
    form: FormGroup<AppointmentForm>,
    updatePageValid: (isValid: boolean) => void,
    step: AppointmentSteps,
  ): void {
    pagesValidation[AppointmentSteps.infoScreen] = { valid: true };
    pagesValidation[AppointmentSteps.pmrAccess] = {
      valid: form.get('pmrAccess').valid,
    };
    pagesValidation[AppointmentSteps.location] = { valid: true };
    pagesValidation[AppointmentSteps.carto] = { valid: true };
    pagesValidation[AppointmentSteps.makeAppointment] = {
      valid: form.get('socialWorkerId').valid && form.get('structureRDV').valid,
    };
    pagesValidation[AppointmentSteps.mediationBeneficiaryInfo] = {
      valid:
        form.get('name').valid &&
        form.get('surname').valid &&
        (form.get('phone').valid || form.get('email').valid) &&
        (!form.get('phone').value || form.get('phone').valid) &&
        (!form.get('email').value || form.get('email').valid),
    };
    pagesValidation[AppointmentSteps.structureOrientator] = {
      valid: this.isStructureOrientatorFormValid(form.get('structureOrientator') as FormGroup),
    };
    pagesValidation[AppointmentSteps.orientationRecap] = { valid: true };
    pagesValidation[AppointmentSteps.rdvEnd] = { valid: true };
    pagesValidation[AppointmentSteps.orientationPrint] = { valid: true };

    updatePageValid(pagesValidation[step].valid);
  }

  public setValidationsEquipmentForm(
    pagesValidation: any[],
    filtersForm: FormGroup<FiltersForm>,
    updatePageValid: (isValid: boolean) => void,
    step: CommonSteps,
  ): void {
    pagesValidation[CommonSteps.filtersChoice] = {
      valid: filtersForm.get('filters').value.length > 0,
    };
    updatePageValid(pagesValidation[step].valid);
  }

  public manuallySetOfPmr(filters: Filter[], event: { name: string; value: boolean }): void {
    // Handle special PMR access case
    if (event.name === 'pmrAccess') {
      const filterIndex = filters.findIndex((f) => f.name === 'pmrAccess');

      if (event.value) {
        if (filterIndex === -1) {
          filters.push(new Filter('pmrAccess', 'True', 'Accessible PMR'));
        } // If filter already exists with value true, do nothing
      } else if (filterIndex !== -1) {
        // If filter exists, remove it
        filters.splice(filterIndex, 1);
      }
    }
  }

  public async rdvDisableItemsNotOffered(
    modules: Module[],
    orientationService: OrientationService,
    category: string,
  ): Promise<void> {
    let personalOffers: PersonalOffer[] = [];
    let offersEnabled: string[] = [];

    // If rdv from structure, get personalOffers of the structure
    if (orientationService.rdvStructure) {
      personalOffers = orientationService.rdvStructure.personalOffers;
    }

    // If rdv from annuaire, get offers of the social worker (pre-populated in OnlineDemarchComponent)
    if (orientationService.rdvUser) {
      personalOffers = orientationService.rdvUser.personalOffersPopulated;
    }

    // Get offers to enable
    personalOffers.forEach((personalOffer) => {
      offersEnabled = offersEnabled.concat(personalOffer.categories[category]);
    });

    // Disable items not offered
    modules.forEach((m) => (m.disabled = !offersEnabled.includes(m.id)));
  }
}
